@@ -1,32 +1,12 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 Jochen Becher
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 Jochen Becher
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "classitem.h"
 
 #include "qmt/controller/namecontroller.h"
 #include "qmt/diagram/dclass.h"
+#include "qmt/diagram/dinheritance.h"
+#include "qmt/diagram_controller/diagramcontroller.h"
 #include "qmt/diagram_scene/diagramsceneconstants.h"
 #include "qmt/diagram_scene/diagramscenemodel.h"
 #include "qmt/diagram_scene/parts/contextlabelitem.h"
@@ -40,6 +20,8 @@
 #include "qmt/infrastructure/qmtassert.h"
 #include "qmt/model/mclass.h"
 #include "qmt/model/mclassmember.h"
+#include "qmt/model/massociation.h"
+#include "qmt/model/minheritance.h"
 #include "qmt/model_controller/modelcontroller.h"
 #include "qmt/stereotype/stereotypecontroller.h"
 #include "qmt/stereotype/stereotypeicon.h"
@@ -48,6 +30,8 @@
 #include "qmt/tasks/diagramscenecontroller.h"
 #include "qmt/tasks/ielementtasks.h"
 
+#include <utils/algorithm.h>
+
 #include <QGraphicsScene>
 #include <QGraphicsRectItem>
 #include <QGraphicsSimpleTextItem>
@@ -59,15 +43,23 @@
 
 #include <algorithm>
 
+#include <qmt/stereotype/customrelation.h>
+
+#include "../../modelinglibtr.h"
+
 namespace qmt {
 
+static const char ASSOCIATION[] = "association";
+static const char INHERITANCE[] = "inheritance";
+
 static const qreal MINIMUM_AUTO_WIDTH = 80.0;
 static const qreal MINIMUM_AUTO_HEIGHT = 60.0;
 static const qreal BODY_VERT_BORDER = 4.0;
 static const qreal BODY_HORIZ_BORDER = 4.0;
+static const qreal BODY_HORIZ_DISTANCE = 4.0;
 
 ClassItem::ClassItem(DClass *klass, DiagramSceneModel *diagramSceneModel, QGraphicsItem *parent)
-    : ObjectItem(klass, diagramSceneModel, parent)
+    : ObjectItem("class", klass, diagramSceneModel, parent)
 {
 }
 
@@ -82,7 +74,7 @@ void ClassItem::update()
     updateStereotypeIconDisplay();
 
     auto diagramClass = dynamic_cast<DClass *>(object());
-    QMT_CHECK(diagramClass);
+    QMT_ASSERT(diagramClass, return);
 
     const Style *style = adaptedStyle(stereotypeIconId());
 
@@ -93,39 +85,67 @@ void ClassItem::update()
         m_methodsText.clear();
     }
 
-    // custom icon
-    if (stereotypeIconDisplay() == StereotypeIcon::DisplayIcon) {
-        if (!m_customIcon)
-            m_customIcon = new CustomIconItem(diagramSceneModel(), this);
-        m_customIcon->setStereotypeIconId(stereotypeIconId());
-        m_customIcon->setBaseSize(stereotypeIconMinimumSize(m_customIcon->stereotypeIcon(), CUSTOM_ICON_MINIMUM_AUTO_WIDTH, CUSTOM_ICON_MINIMUM_AUTO_HEIGHT));
-        m_customIcon->setBrush(style->fillBrush());
-        m_customIcon->setPen(style->outerLinePen());
-        m_customIcon->setZValue(SHAPE_ZVALUE);
-    } else if (m_customIcon) {
-        m_customIcon->scene()->removeItem(m_customIcon);
-        delete m_customIcon;
-        m_customIcon = 0;
-    }
+    updateCustomIcon(style);
 
     // shape
-    if (!m_customIcon) {
+    if (!customIconItem()) {
         if (!m_shape)
             m_shape = new QGraphicsRectItem(this);
         m_shape->setBrush(style->fillBrush());
         m_shape->setPen(style->outerLinePen());
         m_shape->setZValue(SHAPE_ZVALUE);
-    } else if (m_shape){
+    } else if (m_shape) {
         m_shape->scene()->removeItem(m_shape);
         delete m_shape;
-        m_shape = 0;
+        m_shape = nullptr;
     }
 
     // stereotypes
     updateStereotypes(stereotypeIconId(), stereotypeIconDisplay(), style);
 
+    // base classes
+    DiagramController *diagramController = diagramSceneModel()->diagramController();
+    ModelController *modelController = diagramController->modelController();
+    MClass *klass = modelController->findElement<MClass>(diagramClass->modelUid());
+    if (klass) {
+        // TODO cache relation's Uids and check for change
+        QStringList baseClasses;
+        for (const auto &handle : klass->relations()) {
+            if (MInheritance *mInheritance = dynamic_cast<MInheritance *>(handle.target())) {
+                if (mInheritance->base() != klass->uid()) {
+                    DInheritance *dInheritance = diagramController->findDelegate<DInheritance>(mInheritance, diagramSceneModel()->diagram());
+                    if (dInheritance) {
+                        DClass *dBaseClass = diagramController->findElement<DClass>(dInheritance->base(), diagramSceneModel()->diagram());
+                        if (dBaseClass)
+                            continue;
+                    }
+                    MClass *mBaseClass = diagramController->modelController()->findElement<MClass>(mInheritance->base());
+                    if (mBaseClass) {
+                        QString baseClassName;
+                        baseClassName += mBaseClass->name();
+                        baseClasses.append(baseClassName);
+                    }
+                }
+            }
+        }
+        if (!baseClasses.isEmpty()) {
+            if (!m_baseClasses)
+                m_baseClasses = new QGraphicsSimpleTextItem(this);
+            QFont font = style->smallFont();
+            font.setItalic(true);
+            m_baseClasses->setFont(font);
+            m_baseClasses->setBrush(style->textBrush());
+            m_baseClasses->setText(baseClasses.join('\n'));
+        } else if (m_baseClasses) {
+            if (m_baseClasses->scene())
+                m_baseClasses->scene()->removeItem(m_baseClasses);
+            delete m_baseClasses;
+            m_baseClasses = nullptr;
+        }
+    }
+
     // namespace
-    if (!diagramClass->umlNamespace().isEmpty()) {
+    if (!suppressTextDisplay() && !diagramClass->umlNamespace().isEmpty()) {
         if (!m_namespace)
             m_namespace = new QGraphicsSimpleTextItem(this);
         m_namespace->setFont(style->smallFont());
@@ -134,14 +154,14 @@ void ClassItem::update()
     } else if (m_namespace) {
         m_namespace->scene()->removeItem(m_namespace);
         delete m_namespace;
-        m_namespace = 0;
+        m_namespace = nullptr;
     }
 
     // class name
     updateNameItem(style);
 
     // context
-    if (showContext()) {
+    if (!suppressTextDisplay() && showContext()) {
         if (!m_contextLabel)
             m_contextLabel = new ContextLabelItem(this);
         m_contextLabel->setFont(style->smallFont());
@@ -150,11 +170,11 @@ void ClassItem::update()
     } else if (m_contextLabel) {
         m_contextLabel->scene()->removeItem(m_contextLabel);
         delete m_contextLabel;
-        m_contextLabel = 0;
+        m_contextLabel = nullptr;
     }
 
     // attributes separator
-    if (m_shape || !m_attributesText.isEmpty() || !m_methodsText.isEmpty()) {
+    if (m_shape || (!suppressTextDisplay() && (!m_attributesText.isEmpty() || !m_methodsText.isEmpty()))) {
         if (!m_attributesSeparator)
             m_attributesSeparator = new QGraphicsLineItem(this);
         m_attributesSeparator->setPen(style->innerLinePen());
@@ -162,11 +182,11 @@ void ClassItem::update()
     } else if (m_attributesSeparator) {
         m_attributesSeparator->scene()->removeItem(m_attributesSeparator);
         delete m_attributesSeparator;
-        m_attributesSeparator = 0;
+        m_attributesSeparator = nullptr;
     }
 
     // attributes
-    if (!m_attributesText.isEmpty()) {
+    if (!suppressTextDisplay() && !m_attributesText.isEmpty()) {
         if (!m_attributes)
             m_attributes = new QGraphicsTextItem(this);
         m_attributes->setFont(style->normalFont());
@@ -176,11 +196,11 @@ void ClassItem::update()
     } else if (m_attributes) {
         m_attributes->scene()->removeItem(m_attributes);
         delete m_attributes;
-        m_attributes = 0;
+        m_attributes = nullptr;
     }
 
     // methods separator
-    if (m_shape || !m_attributesText.isEmpty() || !m_methodsText.isEmpty()) {
+    if (m_shape || (!suppressTextDisplay() && (!m_attributesText.isEmpty() || !m_methodsText.isEmpty()))) {
         if (!m_methodsSeparator)
             m_methodsSeparator = new QGraphicsLineItem(this);
         m_methodsSeparator->setPen(style->innerLinePen());
@@ -188,11 +208,11 @@ void ClassItem::update()
     } else if (m_methodsSeparator) {
         m_methodsSeparator->scene()->removeItem(m_methodsSeparator);
         delete m_methodsSeparator;
-        m_methodsSeparator = 0;
+        m_methodsSeparator = nullptr;
     }
 
     // methods
-    if (!m_methodsText.isEmpty()) {
+    if (!suppressTextDisplay() && !m_methodsText.isEmpty()) {
         if (!m_methods)
             m_methods = new QGraphicsTextItem(this);
         m_methods->setFont(style->normalFont());
@@ -202,7 +222,7 @@ void ClassItem::update()
     } else if (m_methods) {
         m_methods->scene()->removeItem(m_methods);
         delete m_methods;
-        m_methods = 0;
+        m_methods = nullptr;
     }
 
     // template parameters
@@ -223,35 +243,36 @@ void ClassItem::update()
     } else if (m_templateParameterBox) {
         m_templateParameterBox->scene()->removeItem(m_templateParameterBox);
         delete m_templateParameterBox;
-        m_templateParameterBox = 0;
-    }
-
-    updateSelectionMarker(m_customIcon);
-
-    // relation starters
-    if (isFocusSelected()) {
-        if (!m_relationStarter) {
-            m_relationStarter = new RelationStarter(this, diagramSceneModel(), 0);
-            scene()->addItem(m_relationStarter);
-            m_relationStarter->setZValue(RELATION_STARTER_ZVALUE);
-            m_relationStarter->addArrow(QLatin1String("inheritance"), ArrowItem::ShaftSolid, ArrowItem::HeadTriangle);
-            m_relationStarter->addArrow(QLatin1String("dependency"), ArrowItem::ShaftDashed, ArrowItem::HeadOpen);
-            m_relationStarter->addArrow(QLatin1String("association"), ArrowItem::ShaftSolid, ArrowItem::HeadFilledTriangle);
-        }
-    } else if (m_relationStarter) {
-        scene()->removeItem(m_relationStarter);
-        delete m_relationStarter;
-        m_relationStarter = 0;
+        m_templateParameterBox = nullptr;
     }
 
+    updateSelectionMarker(customIconItem());
+    updateRelationStarter();
     updateAlignmentButtons();
     updateGeometry();
 }
 
 bool ClassItem::intersectShapeWithLine(const QLineF &line, QPointF *intersectionPoint, QLineF *intersectionLine) const
 {
+    if (customIconItem()) {
+        QList<QPolygonF> polygons = customIconItem()->outline();
+        for (int i = 0; i < polygons.size(); ++i)
+            polygons[i].translate(object()->pos() + object()->rect().topLeft());
+        if (shapeIcon().textAlignment() == qmt::StereotypeIcon::TextalignBelow) {
+            if (nameItem()) {
+                QPolygonF polygon(nameItem()->boundingRect());
+                polygon.translate(object()->pos() + nameItem()->pos());
+                polygons.append(polygon);
+            }
+            if (m_contextLabel) {
+                QPolygonF polygon(m_contextLabel->boundingRect());
+                polygon.translate(object()->pos() + m_contextLabel->pos());
+                polygons.append(polygon);
+            }
+        }
+        return GeometryUtilities::intersect(polygons, line, nullptr, intersectionPoint, intersectionLine);
+    }
     QPolygonF polygon;
-    // TODO if m_customIcon then use that shape + label's shape as intersection path
     QRectF rect = object()->rect();
     rect.translate(object()->pos());
     polygon << rect.topLeft() << rect.topRight() << rect.bottomRight() << rect.bottomLeft() << rect.topLeft();
@@ -263,55 +284,101 @@ QSizeF ClassItem::minimumSize() const
     return calcMinimumGeometry();
 }
 
-QPointF ClassItem::relationStartPos() const
+void ClassItem::relationDrawn(const QString &id, ObjectItem *targetItem, const QList<QPointF> &intermediatePoints)
 {
-    return pos();
-}
-
-void ClassItem::relationDrawn(const QString &id, const QPointF &toScenePos, const QList<QPointF> &intermediatePoints)
-{
-    DElement *targetElement = diagramSceneModel()->findTopmostElement(toScenePos);
-    if (targetElement) {
-        if (id == QLatin1String("inheritance")) {
-            auto baseClass = dynamic_cast<DClass *>(targetElement);
-            if (baseClass) {
-                auto derivedClass = dynamic_cast<DClass *>(object());
-                QMT_CHECK(derivedClass);
-                diagramSceneModel()->diagramSceneController()->createInheritance(derivedClass, baseClass, intermediatePoints, diagramSceneModel()->diagram());
+    DiagramSceneController *diagramSceneController = diagramSceneModel()->diagramSceneController();
+    if (id == INHERITANCE) {
+        auto baseClass = dynamic_cast<DClass *>(targetItem->object());
+        if (baseClass) {
+            auto derivedClass = dynamic_cast<DClass *>(object());
+            QMT_ASSERT(derivedClass, return);
+            diagramSceneController->createInheritance(derivedClass, baseClass, intermediatePoints, diagramSceneModel()->diagram());
+        }
+        return;
+    } else if (id == ASSOCIATION) {
+        auto associatedClass = dynamic_cast<DClass *>(targetItem->object());
+        if (associatedClass) {
+            auto derivedClass = dynamic_cast<DClass *>(object());
+            QMT_ASSERT(derivedClass, return);
+            diagramSceneController->createAssociation(derivedClass, associatedClass, intermediatePoints, diagramSceneModel()->diagram());
+        }
+        return;
+    } else {
+        StereotypeController *stereotypeController = diagramSceneModel()->stereotypeController();
+        CustomRelation customRelation = stereotypeController->findCustomRelation(id);
+        if (!customRelation.isNull()) {
+            switch (customRelation.element()) {
+            case CustomRelation::Element::Inheritance:
+            {
+                auto baseClass = dynamic_cast<DClass *>(targetItem->object());
+                if (baseClass) {
+                    auto derivedClass = dynamic_cast<DClass *>(object());
+                    QMT_ASSERT(derivedClass, return);
+                    diagramSceneController->createInheritance(derivedClass, baseClass, intermediatePoints, diagramSceneModel()->diagram());
+                }
+                return;
             }
-        } else if (id == QLatin1String("dependency")) {
-            auto dependantObject = dynamic_cast<DObject *>(targetElement);
-            if (dependantObject)
-                diagramSceneModel()->diagramSceneController()->createDependency(object(), dependantObject, intermediatePoints, diagramSceneModel()->diagram());
-        } else if (id == QLatin1String("association")) {
-            auto assoziatedClass = dynamic_cast<DClass *>(targetElement);
-            if (assoziatedClass) {
-                auto derivedClass = dynamic_cast<DClass *>(object());
-                QMT_CHECK(derivedClass);
-                diagramSceneModel()->diagramSceneController()->createAssociation(derivedClass, assoziatedClass, intermediatePoints, diagramSceneModel()->diagram());
+            case CustomRelation::Element::Association:
+            {
+                auto assoziatedClass = dynamic_cast<DClass *>(targetItem->object());
+                if (assoziatedClass) {
+                    auto derivedClass = dynamic_cast<DClass *>(object());
+                    QMT_ASSERT(derivedClass, return);
+                    diagramSceneController->createAssociation(derivedClass, assoziatedClass,
+                                intermediatePoints, diagramSceneModel()->diagram(),
+                                [diagramSceneController, customRelation]
+                                (MAssociation *mAssociation, DAssociation *dAssociation) {
+                        if (mAssociation && dAssociation) {
+                            static const QHash<CustomRelation::Relationship, MAssociationEnd::Kind> relationship2KindMap = {
+                                { CustomRelation::Relationship::Association, MAssociationEnd::Association },
+                                { CustomRelation::Relationship::Aggregation, MAssociationEnd::Aggregation },
+                                { CustomRelation::Relationship::Composition, MAssociationEnd::Composition } };
+                            diagramSceneController->modelController()->startUpdateRelation(mAssociation);
+                            mAssociation->setStereotypes(Utils::toList(customRelation.stereotypes()));
+                            mAssociation->setName(customRelation.name());
+                            MAssociationEnd endA;
+                            endA.setCardinality(customRelation.endA().cardinality());
+                            endA.setKind(relationship2KindMap.value(customRelation.endA().relationship()));
+                            endA.setName(customRelation.endA().role());
+                            endA.setNavigable(customRelation.endA().navigable());
+                            mAssociation->setEndA(endA);
+                            MAssociationEnd endB;
+                            endB.setCardinality(customRelation.endB().cardinality());
+                            endB.setKind(relationship2KindMap.value(customRelation.endB().relationship()));
+                            endB.setName(customRelation.endB().role());
+                            endB.setNavigable(customRelation.endB().navigable());
+                            mAssociation->setEndB(endB);
+                            diagramSceneController->modelController()->finishUpdateRelation(mAssociation, false);
+                        }
+                    });
+                }
+                return;
+            }
+            case CustomRelation::Element::Dependency:
+            case CustomRelation::Element::Relation:
+                // fall thru
+                break;
             }
         }
     }
+    ObjectItem::relationDrawn(id, targetItem, intermediatePoints);
 }
 
 bool ClassItem::extendContextMenu(QMenu *menu)
 {
     bool extended = false;
     if (diagramSceneModel()->diagramSceneController()->elementTasks()->hasClassDefinition(object(), diagramSceneModel()->diagram())) {
-        menu->addAction(new ContextMenuAction(tr("Show Definition"), QStringLiteral("showDefinition"), menu));
+        menu->addAction(new ContextMenuAction(Tr::tr("Show Definition"), "showDefinition", menu));
         extended = true;
     }
     return extended;
 }
 
-bool ClassItem::handleSelectedContextMenuAction(QAction *action)
+bool ClassItem::handleSelectedContextMenuAction(const QString &id)
 {
-    auto klassAction = dynamic_cast<ContextMenuAction *>(action);
-    if (klassAction) {
-        if (klassAction->id() == QStringLiteral("showDefinition")) {
-            diagramSceneModel()->diagramSceneController()->elementTasks()->openClassDefinition(object(), diagramSceneModel()->diagram());
-            return true;
-        }
+    if (id == "showDefinition") {
+        diagramSceneModel()->diagramSceneController()->elementTasks()->openClassDefinition(object(), diagramSceneModel()->diagram());
+        return true;
     }
     return false;
 }
@@ -319,14 +386,14 @@ bool ClassItem::handleSelectedContextMenuAction(QAction *action)
 QString ClassItem::buildDisplayName() const
 {
     auto diagramClass = dynamic_cast<DClass *>(object());
-    QMT_CHECK(diagramClass);
+    QMT_ASSERT(diagramClass, return QString());
 
     QString name;
     if (templateDisplay() == DClass::TemplateName && !diagramClass->templateParameters().isEmpty()) {
         name = object()->name();
         name += QLatin1Char('<');
         bool first = true;
-        foreach (const QString &p, diagramClass->templateParameters()) {
+        for (const QString &p : diagramClass->templateParameters()) {
             if (!first)
                 name += QLatin1Char(',');
             name += p;
@@ -345,9 +412,9 @@ void ClassItem::setFromDisplayName(const QString &displayName)
         QString name;
         QStringList templateParameters;
         // NOTE namespace is ignored because it has its own edit field
-        if (NameController::parseClassName(displayName, 0, &name, &templateParameters)) {
+        if (NameController::parseClassName(displayName, nullptr, &name, &templateParameters)) {
             auto diagramClass = dynamic_cast<DClass *>(object());
-            QMT_CHECK(diagramClass);
+            QMT_ASSERT(diagramClass, return);
             ModelController *modelController = diagramSceneModel()->diagramSceneController()->modelController();
             MClass *mklass = modelController->findObject<MClass>(diagramClass->modelUid());
             if (mklass && (name != mklass->name() || templateParameters != mklass->templateParameters())) {
@@ -362,14 +429,74 @@ void ClassItem::setFromDisplayName(const QString &displayName)
     }
 }
 
+void ClassItem::addRelationStarterTool(const QString &id)
+{
+    if (id == INHERITANCE)
+        relationStarter()->addArrow(INHERITANCE, ArrowItem::ShaftSolid,
+                                    ArrowItem::HeadNone, ArrowItem::HeadTriangle,
+                                    Tr::tr("Inheritance"));
+    else if (id == ASSOCIATION)
+        relationStarter()->addArrow(ASSOCIATION, ArrowItem::ShaftSolid,
+                                    ArrowItem::HeadNone, ArrowItem::HeadFilledTriangle,
+                                    Tr::tr("Association"));
+    else
+        ObjectItem::addRelationStarterTool(id);
+}
+
+void ClassItem::addRelationStarterTool(const CustomRelation &customRelation)
+{
+    ArrowItem::Shaft shaft = ArrowItem::ShaftSolid;
+    ArrowItem::Head headStart = ArrowItem::HeadNone;
+    ArrowItem::Head headEnd = ArrowItem::HeadNone;
+    switch (customRelation.element()) {
+    case CustomRelation::Element::Inheritance:
+        shaft = ArrowItem::ShaftSolid;
+        headEnd = ArrowItem::HeadTriangle;
+        break;
+    case CustomRelation::Element::Association:
+        switch (customRelation.endA().relationship()) {
+        case CustomRelation::Relationship::Association:
+            if (customRelation.endA().navigable() && customRelation.endB().navigable()) {
+                headStart = ArrowItem::HeadNone;
+                headEnd = ArrowItem::HeadNone;
+            } else if (customRelation.endA().navigable()) {
+                headStart = ArrowItem::HeadFilledTriangle;
+            } else {
+                headEnd = ArrowItem::HeadFilledTriangle;
+            }
+            break;
+        case CustomRelation::Relationship::Aggregation:
+            headStart = ArrowItem::HeadDiamond;
+            break;
+        case CustomRelation::Relationship::Composition:
+            headStart = ArrowItem::HeadFilledDiamond;
+            break;
+        }
+        break;
+    case CustomRelation::Element::Dependency:
+    case CustomRelation::Element::Relation:
+        ObjectItem::addRelationStarterTool(customRelation);
+        return;
+    }
+    relationStarter()->addArrow(customRelation.id(), shaft, headStart, headEnd,
+                                customRelation.title());
+}
+
+void ClassItem::addStandardRelationStarterTools()
+{
+    ObjectItem::addStandardRelationStarterTools();
+    addRelationStarterTool(INHERITANCE);
+    addRelationStarterTool(ASSOCIATION);
+}
+
 DClass::TemplateDisplay ClassItem::templateDisplay() const
 {
     auto diagramClass = dynamic_cast<DClass *>(object());
-    QMT_CHECK(diagramClass);
+    QMT_ASSERT(diagramClass, return DClass::TemplateSmart);
 
     DClass::TemplateDisplay templateDisplay = diagramClass->templateDisplay();
     if (templateDisplay == DClass::TemplateSmart) {
-        if (m_customIcon)
+        if (customIconItem())
             templateDisplay = DClass::TemplateName;
         else
             templateDisplay = DClass::TemplateBox;
@@ -382,26 +509,38 @@ QSizeF ClassItem::calcMinimumGeometry() const
     double width = 0.0;
     double height = 0.0;
 
-    if (m_customIcon) {
-        return stereotypeIconMinimumSize(m_customIcon->stereotypeIcon(),
-                                         CUSTOM_ICON_MINIMUM_AUTO_WIDTH, CUSTOM_ICON_MINIMUM_AUTO_HEIGHT);
+    if (customIconItem()) {
+        QSizeF sz = customIconItemMinimumSize(customIconItem(),
+                                              CUSTOM_ICON_MINIMUM_AUTO_WIDTH, CUSTOM_ICON_MINIMUM_AUTO_HEIGHT);
+        if (shapeIcon().textAlignment() != qmt::StereotypeIcon::TextalignTop
+                && shapeIcon().textAlignment() != qmt::StereotypeIcon::TextalignCenter)
+            return sz;
+        width = sz.width();
     }
 
     height += BODY_VERT_BORDER;
+    double top_row_width = BODY_HORIZ_DISTANCE;
+    double top_row_height = 0;
     if (CustomIconItem *stereotypeIconItem = this->stereotypeIconItem()) {
-        width = std::max(width, stereotypeIconItem->boundingRect().width() + 2 * BODY_HORIZ_BORDER);
-        height += stereotypeIconItem->boundingRect().height();
+        top_row_width += stereotypeIconItem->boundingRect().width();
+        top_row_height = stereotypeIconItem->boundingRect().height();
     }
+    if (m_baseClasses) {
+        top_row_width += m_baseClasses->boundingRect().width();
+        top_row_height = std::max(top_row_height, m_baseClasses->boundingRect().height());
+    }
+    width = std::max(width, top_row_width);
+    height += top_row_height;
     if (StereotypesItem *stereotypesItem = this->stereotypesItem()) {
-        width = std::max(width, stereotypesItem->boundingRect().width() + 2 * BODY_HORIZ_BORDER);
+        width = std::max(width, stereotypesItem->boundingRect().width());
         height += stereotypesItem->boundingRect().height();
     }
     if (m_namespace) {
-        width = std::max(width, m_namespace->boundingRect().width() + 2 * BODY_HORIZ_BORDER);
+        width = std::max(width, m_namespace->boundingRect().width());
         height += m_namespace->boundingRect().height();
     }
     if (nameItem()) {
-        width = std::max(width, nameItem()->boundingRect().width() + 2 * BODY_HORIZ_BORDER);
+        width = std::max(width, nameItem()->boundingRect().width());
         height += nameItem()->boundingRect().height();
     }
     if (m_contextLabel)
@@ -409,17 +548,19 @@ QSizeF ClassItem::calcMinimumGeometry() const
     if (m_attributesSeparator)
         height += 8.0;
     if (m_attributes) {
-        width = std::max(width, m_attributes->boundingRect().width() + 2 * BODY_HORIZ_BORDER);
+        width = std::max(width, m_attributes->boundingRect().width());
         height += m_attributes->boundingRect().height();
     }
     if (m_methodsSeparator)
         height += 8.0;
     if (m_methods) {
-        width = std::max(width, m_methods->boundingRect().width() + 2 * BODY_HORIZ_BORDER);
+        width = std::max(width, m_methods->boundingRect().width());
         height += m_methods->boundingRect().height();
     }
     height += BODY_VERT_BORDER;
 
+    width = BODY_HORIZ_BORDER + width + BODY_HORIZ_BORDER;
+
     return GeometryUtilities::ensureMinimumRasterSize(QSizeF(width, height), 2 * RASTER_WIDTH, 2 * RASTER_HEIGHT);
 }
 
@@ -436,12 +577,7 @@ void ClassItem::updateGeometry()
     height = geometry.height();
 
     if (object()->isAutoSized()) {
-        if (!m_customIcon) {
-            if (width < MINIMUM_AUTO_WIDTH)
-                width = MINIMUM_AUTO_WIDTH;
-            if (height < MINIMUM_AUTO_HEIGHT)
-                height = MINIMUM_AUTO_HEIGHT;
-        }
+        correctAutoSize(customIconItem(), width, height, MINIMUM_AUTO_WIDTH, MINIMUM_AUTO_HEIGHT);
     } else {
         QRectF rect = object()->rect();
         if (rect.width() > width)
@@ -466,20 +602,53 @@ void ClassItem::updateGeometry()
     // a backup for the graphics item used for manual resized and persistency.
     object()->setRect(rect);
 
-    if (m_customIcon) {
-        m_customIcon->setPos(left, top);
-        m_customIcon->setActualSize(QSizeF(width, height));
-        y += height;
+    if (customIconItem()) {
+        customIconItem()->setPos(left, top);
+        customIconItem()->setActualSize(QSizeF(width, height));
     }
 
     if (m_shape)
         m_shape->setRect(rect);
 
-    y += BODY_VERT_BORDER;
+    if (customIconItem()) {
+        switch (shapeIcon().textAlignment()) {
+        case qmt::StereotypeIcon::TextalignBelow:
+            y += height + BODY_VERT_BORDER;
+            break;
+        case qmt::StereotypeIcon::TextalignCenter:
+        {
+            double h = 0.0;
+            if (CustomIconItem *stereotypeIconItem = this->stereotypeIconItem())
+                h += stereotypeIconItem->boundingRect().height();
+            if (StereotypesItem *stereotypesItem = this->stereotypesItem())
+                h += stereotypesItem->boundingRect().height();
+            if (nameItem())
+                h += nameItem()->boundingRect().height();
+            if (m_contextLabel)
+                h += m_contextLabel->height();
+            y = top + (height - h) / 2.0;
+            break;
+        }
+        case qmt::StereotypeIcon::TextalignNone:
+            // nothing to do
+            break;
+        case qmt::StereotypeIcon::TextalignTop:
+            y += BODY_VERT_BORDER;
+            break;
+        }
+    } else {
+        y += BODY_VERT_BORDER;
+    }
+    double first_row_height = 0;
+    if (m_baseClasses) {
+        m_baseClasses->setPos(left + BODY_HORIZ_BORDER, y);
+        first_row_height = m_baseClasses->boundingRect().height();
+    }
     if (CustomIconItem *stereotypeIconItem = this->stereotypeIconItem()) {
         stereotypeIconItem->setPos(right - stereotypeIconItem->boundingRect().width() - BODY_HORIZ_BORDER, y);
-        y += stereotypeIconItem->boundingRect().height();
+        first_row_height = std::max(first_row_height, stereotypeIconItem->boundingRect().height());
     }
+    y += first_row_height;
     if (StereotypesItem *stereotypesItem = this->stereotypesItem()) {
         stereotypesItem->setPos(-stereotypesItem->boundingRect().width() / 2.0, y);
         y += stereotypesItem->boundingRect().height();
@@ -493,7 +662,7 @@ void ClassItem::updateGeometry()
         y += nameItem()->boundingRect().height();
     }
     if (m_contextLabel) {
-        if (m_customIcon)
+        if (customIconItem())
             m_contextLabel->resetMaxWidth();
         else
             m_contextLabel->setMaxWidth(width - 2 * BODY_HORIZ_BORDER);
@@ -506,7 +675,7 @@ void ClassItem::updateGeometry()
         y += 8.0;
     }
     if (m_attributes) {
-        if (m_customIcon)
+        if (customIconItem())
             m_attributes->setPos(-m_attributes->boundingRect().width() / 2.0, y);
         else
             m_attributes->setPos(left + BODY_HORIZ_BORDER, y);
@@ -518,7 +687,7 @@ void ClassItem::updateGeometry()
         y += 8.0;
     }
     if (m_methods) {
-        if (m_customIcon)
+        if (customIconItem())
             m_methods->setPos(-m_methods->boundingRect().width() / 2.0, y);
         else
             m_methods->setPos(left + BODY_HORIZ_BORDER, y);
@@ -538,10 +707,7 @@ void ClassItem::updateGeometry()
     }
 
     updateSelectionMarkerGeometry(rect);
-
-    if (m_relationStarter)
-        m_relationStarter->setPos(mapToScene(QPointF(right + 8.0, top)));
-
+    updateRelationStarterGeometry(rect);
     updateAlignmentButtonsGeometry(rect);
     updateDepth();
 }
@@ -558,17 +724,17 @@ void ClassItem::updateMembers(const Style *style)
     QString attributesGroup;
     QString methodsGroup;
 
-    MClassMember::Visibility *currentVisibility = 0;
-    QString *currentGroup = 0;
-    QString *text = 0;
+    MClassMember::Visibility *currentVisibility = nullptr;
+    QString *currentGroup = nullptr;
+    QString *text = nullptr;
 
     auto dclass = dynamic_cast<DClass *>(object());
-    QMT_CHECK(dclass);
+    QMT_ASSERT(dclass, return);
 
-    foreach (const MClassMember &member, dclass->members()) {
+    for (const MClassMember &member : dclass->members()) {
         switch (member.memberType()) {
         case MClassMember::MemberUndefined:
-            QMT_CHECK(false);
+            QMT_ASSERT(false, return);
             break;
         case MClassMember::MemberAttribute:
             currentVisibility = &attributesVisibility;
@@ -582,21 +748,22 @@ void ClassItem::updateMembers(const Style *style)
             break;
         }
 
+        bool needBr = false;
         if (text && !text->isEmpty())
-            *text += QStringLiteral("<br/>");
+            needBr = true;
 
-        bool addNewline = false;
-        bool addSpace = false;
         if (currentVisibility)
             *currentVisibility = member.visibility();
         if (currentGroup && member.group() != *currentGroup) {
-            *text += QString(QStringLiteral("[%1]")).arg(member.group());
-            addNewline = true;
+            needBr = false;
+            *text += QString("<p style=\"padding:0;margin-top:6;margin-bottom:0;\"><b>[%1]</b></p>").arg(member.group());
             *currentGroup = member.group();
         }
-        if (addNewline)
-            *text += QStringLiteral("<br/>");
 
+        if (needBr)
+            *text += "<br/>";
+
+        bool addSpace = false;
         bool haveSignal = false;
         bool haveSlot = false;
         if (member.visibility() != MClassMember::VisibilityUndefined) {
@@ -605,34 +772,34 @@ void ClassItem::updateMembers(const Style *style)
             case MClassMember::VisibilityUndefined:
                 break;
             case MClassMember::VisibilityPublic:
-                vis = QStringLiteral("+");
+                vis = "+";
                 addSpace = true;
                 break;
             case MClassMember::VisibilityProtected:
-                vis = QStringLiteral("#");
+                vis = "#";
                 addSpace = true;
                 break;
             case MClassMember::VisibilityPrivate:
-                vis = QStringLiteral("-");
+                vis = "-";
                 addSpace = true;
                 break;
             case MClassMember::VisibilitySignals:
-                vis = QStringLiteral("&gt;");
+                vis = "&gt;";
                 haveSignal = true;
                 addSpace = true;
                 break;
             case MClassMember::VisibilityPrivateSlots:
-                vis = QStringLiteral("-$");
+                vis = "-$";
                 haveSlot = true;
                 addSpace = true;
                 break;
             case MClassMember::VisibilityProtectedSlots:
-                vis = QStringLiteral("#$");
+                vis = "#$";
                 haveSlot = true;
                 addSpace = true;
                 break;
             case MClassMember::VisibilityPublicSlots:
-                vis = QStringLiteral("+$");
+                vis = "+$";
                 haveSlot = true;
                 addSpace = true;
                 break;
@@ -641,34 +808,34 @@ void ClassItem::updateMembers(const Style *style)
         }
 
         if (member.properties() & MClassMember::PropertyQsignal && !haveSignal) {
-            *text += QStringLiteral("&gt;");
+            *text += "&gt;";
             addSpace = true;
         }
         if (member.properties() & MClassMember::PropertyQslot && !haveSlot) {
-            *text += QStringLiteral("$");
+            *text += "$";
             addSpace = true;
         }
         if (addSpace)
-            *text += QStringLiteral(" ");
+            *text += " ";
         if (member.properties() & MClassMember::PropertyQinvokable)
-            *text += QStringLiteral("invokable ");
+            *text += "invokable ";
         if (!member.stereotypes().isEmpty()) {
             *text += StereotypesItem::format(member.stereotypes());
-            *text += QStringLiteral(" ");
+            *text += " ";
         }
         if (member.properties() & MClassMember::PropertyStatic)
-            *text += QStringLiteral("static ");
+            *text += "static ";
         if (member.properties() & MClassMember::PropertyVirtual)
-            *text += QStringLiteral("virtual ");
+            *text += "virtual ";
         *text += member.declaration().toHtmlEscaped();
         if (member.properties() & MClassMember::PropertyConst)
-            *text += QStringLiteral(" const");
+            *text += " const";
         if (member.properties() & MClassMember::PropertyOverride)
-            *text += QStringLiteral(" override");
+            *text += " override";
         if (member.properties() & MClassMember::PropertyFinal)
-            *text += QStringLiteral(" final");
+            *text += " final";
         if (member.properties() & MClassMember::PropertyAbstract)
-            *text += QStringLiteral(" = 0");
+            *text += " = 0";
     }
 }
 
