@@ -103,37 +103,111 @@ static bool issueListContextMenuEvent(const ItemViewEvent &ev); // impl at botto
 static bool progressListContextMenuEvent(const ItemViewEvent &ev); // impl at bottom
 static void resetFocusToIssuesTable(); // impl at bottom
 
-static std::optional<PathMapping> findPathMappingMatch(const QString &projectName,
-                                                       const Link &link)
+/**
+ * combine local (absolute) path with analysis path
+ * if analysis path is absolute just return the local path
+ * otherwise check whether the local path ends with (parts of) analysis path
+ * and return the combined path or if no match append analysis to local path
+ *
+ * examples:
+ * local                analysis                    result
+ * /a/b/c               /d/e                        /a/b/c
+ * /a/b/c               d/e                         /a/b/c/d/e
+ * /a/b/c               c                           /a/b
+ * /a/b/c               c/d                         /a/b/c/c/d
+ * /a/b/c               b/c/d                       /a/b/c/b/c/d
+ * /a/b/c               b                           /a/b/c/b
+ * /a/b/c/d             b/d/e                       /a/b/c/d/b/d/e
+ */
+static FilePath combinedPaths(const FilePath &localPath, const FilePath &analysisPath)
 {
-    QTC_ASSERT(!projectName.isEmpty(), return std::nullopt);
+    if (analysisPath.isEmpty() || analysisPath.isAbsolutePath())
+        return localPath;
+
+    const QList<QStringView> localComp = localPath.pathComponents();
+    const QList<QStringView> analysisComp = analysisPath.pathComponents();
+    const int localCount = localComp.size();
+    const int analysisCount = analysisComp.size();
+
+    for (int localBwd = localCount - 1; localBwd >= 0; --localBwd) {
+        // do we have a potential start of match
+        if (localComp.at(localBwd) == analysisComp.first()) {
+            // only if all analysis path components do fit
+            if (localBwd + analysisCount < localCount)
+                continue;
+            int fwd = 1;
+            // check whether all analysis path components match
+            for ( ; fwd < analysisCount && localBwd + fwd < localCount; ++fwd) {
+                if (localComp.at(localBwd + fwd) != analysisComp.at(fwd))
+                    break;
+            }
+            // check whether we broke at a mismatch
+            if (fwd != analysisCount && (localBwd + fwd != localCount))
+                continue;
+
+            QList<QStringView> resultPath = localComp.sliced(0, localBwd);
+            if (resultPath.first() == u"/") // hack for UNIX paths
+                resultPath.replace(0, QStringView{});
+            const QStringList pathStrings = Utils::transform(resultPath, &QStringView::toString);
+            return FilePath::fromUserInput(pathStrings.join('/'));
+        }
+    }
+    return localPath.pathAppended(analysisPath.path());
+}
+
+static QList<PathMapping> findPathMappingMatches(const QString &projectName, const Link &link)
+{
+    QList<PathMapping> result;
+    QTC_ASSERT(!projectName.isEmpty(), return result);
+    const bool pathIsAbsolute = link.targetFilePath.isAbsolutePath();
     for (const PathMapping &mapping : settings().validPathMappings()) {
         if (mapping.projectName != projectName)
             continue;
 
-        if (mapping.analysisPath.isEmpty())
-            return mapping;
-
         QString analysis = mapping.analysisPath.path();
         // ensure we use complete paths
-        if (!analysis.endsWith('/'))
+        if (!analysis.isEmpty() && !analysis.endsWith('/'))
             analysis.append('/');
-        if (!link.targetFilePath.startsWith(analysis))
-            continue;
 
-        return mapping;
+        if (pathIsAbsolute) {
+            if (mapping.analysisPath.isEmpty())
+                continue;
+
+            if (!link.targetFilePath.startsWith(analysis))
+                continue;
+        } else {
+            if (mapping.analysisPath.isAbsolutePath())
+                continue;
+
+            if (!analysis.isEmpty() && !link.targetFilePath.startsWith(analysis))
+                continue;
+
+        }
+
+        result << mapping;
     }
-    return std::nullopt;
+    return result;
 }
 
 static FilePath mappedPathForLink(const Link &link)
 {
     if (const std::optional<Dto::ProjectInfoDto> pInfo = projectInfo()) {
-        if (auto mapping = findPathMappingMatch(pInfo->name, link)) {
+        const QList<PathMapping> mappings = findPathMappingMatches(pInfo->name, link);
+        const bool linkIsRelative = link.targetFilePath.isRelativePath();
+        for (const PathMapping &mapping : mappings) {
+            if (linkIsRelative) {
+                // combine local & analysis paths
+                const FilePath localPath = combinedPaths(mapping.localPath, mapping.analysisPath);
+                const FilePath mapped = localPath.pathAppended(link.targetFilePath.path());
+                if (mapped.exists())
+                    return mapped;
+                continue;
+            }
+            // else absolute paths
             std::optional<FilePath> fp = link.targetFilePath.prefixRemoved(
-                        mapping->analysisPath.path());
-            QTC_CHECK(fp);
-            fp = mapping->localPath.pathAppended(fp->path());
+                        mapping.analysisPath.path());
+            QTC_ASSERT(fp, return {});
+            fp = mapping.localPath.pathAppended(fp->path());
             if (fp->exists())
                 return *fp;
         }
@@ -141,6 +215,16 @@ static FilePath mappedPathForLink(const Link &link)
     return {};
 }
 
+static FilePath requestPathMapping(const Link &link, const QString &projectName)
+{
+
+    if (projectName.isEmpty())
+        return {}; // we fail to create a mapping for empty project name
+    if (!handleMissingPathMapping(link.targetFilePath, projectName)) // mapping still invalid
+        return {};
+    return mappedPathForLink(link);
+}
+
 class IssueListItem final : public ListItem
 {
 public:
@@ -175,13 +259,13 @@ public:
                     return link.columns.contains(column);
                 }).link;
 
-                // get the file path either by the open project(s) or by using the path mapping
-                // prefer to use the open project's file instead of some generic approach
-                const FilePath computedPath = findFileForIssuePath(link.targetFilePath);
-                FilePath targetFilePath;
-                if (!computedPath.exists())
-                    targetFilePath = mappedPathForLink(link);
-                link.targetFilePath = targetFilePath.isEmpty() ? computedPath : targetFilePath;
+                FilePath targetFilePath = mappedPathForLink(link);
+                if (targetFilePath.isEmpty()) {
+                    auto pi = projectInfo();
+                    QTC_ASSERT(pi, return false);
+                    targetFilePath = requestPathMapping(link, pi->name);
+                }
+                link.targetFilePath = targetFilePath;
                 if (link.targetFilePath.exists()) {
                     EditorManager::openEditorAt(link);
                     resetFocusToIssuesTable();
@@ -279,7 +363,7 @@ private:
     int m_totalRowCount = 0;
     QStringList m_userNames;
     QStringList m_versionDates;
-    TaskTreeRunner m_taskTreeRunner;
+    SingleTaskTreeRunner m_taskTreeRunner;
     OverlayWidget *m_overlay = nullptr;
     bool m_dashboardListUninitialized = true;
 
@@ -333,6 +417,8 @@ IssuesWidget::IssuesWidget(QWidget *parent)
             return;
         }
 
+        m_issuesView->showProgressIndicator();
+        setFiltersEnabled(false);
         fetchDashboardAndProjectInfo({}, m_dashboardProjects->currentText());
     });
     // row with local build + dashboard, issue types (-> depending on choice, tables below change)
@@ -358,10 +444,7 @@ IssuesWidget::IssuesWidget(QWidget *parent)
     localLayout->addWidget(m_localBuild);
     localLayout->addWidget(m_localDashBoard);
     connect(&settings(), &AxivionSettings::suitePathValidated, this, [this] {
-        const auto info = settings().versionInfo();
-        const bool enable = info && !info->versionNumber.isEmpty()
-                && !hasRunningLocalBuild(m_currentProject);
-        m_localBuild->setEnabled(enable);
+        m_localBuild->setEnabled(!hasRunningLocalBuild(m_currentProject));
         checkForLocalBuildAndUpdate();
     });
     connect(m_localDashBoard, &QToolButton::clicked, this, &IssuesWidget::switchDashboard);
@@ -556,8 +639,27 @@ void IssuesWidget::updateUi(const QString &kind)
             }
         }
     }
-    if (m_currentPrefix.isEmpty())
-        m_currentPrefix = info.issueKinds.size() ? info.issueKinds.front().prefix : QString{};
+    if (m_currentPrefix.isEmpty()) {
+        const QString preferred = settings().defaultIssueKind();
+        const int index =
+                Utils::indexOf(info.issueKinds, [preferred](const Dto::IssueKindInfoDto &dto) {
+            return dto.prefix == preferred;
+        });
+        if (index != -1) {
+            auto kindButton = m_typesButtonGroup->button(index + 1);
+            if (QTC_GUARD(kindButton))
+                kindButton->setChecked(true);
+            m_currentPrefix = preferred;
+        }
+        // if we could not find the preferred kind, fall back to old approach
+        if (m_currentPrefix.isEmpty()) {
+            const int id = m_typesButtonGroup->checkedId();
+            if (id > 0 && id <= int(info.issueKinds.size()))
+                m_currentPrefix = info.issueKinds.at(id - 1).prefix;
+            else
+                m_currentPrefix = info.issueKinds.size() ? info.issueKinds.front().prefix : QString{};
+        }
+    }
     fetchTable();
 }
 
@@ -575,16 +677,20 @@ void IssuesWidget::leaveOrEnterDashboardMode(bool byLocalBuildButton)
 {
     GuardLocker lock(m_signalBlocker);
 
+    checkForLocalBuildAndUpdate();
+
     switch (currentDashboardMode()) {
     case DashboardMode::Global:
         m_versionsStack->setCurrentIndex(int(DashboardMode::Global));
+        m_localDashBoard->setChecked(false);
         if (!byLocalBuildButton) {
             QTC_ASSERT(currentDashboardInfo(), reinitProjectList(m_currentProject); return);
+            m_issuesView->showProgressIndicator();
+            setFiltersEnabled(false);
             fetchDashboardAndProjectInfo({}, m_dashboardProjects->currentText());
             return;
         }
 
-        m_localDashBoard->setChecked(false);
         fetchNamedFilters(DashboardMode::Global);
         fetchTable();
         break;
@@ -685,6 +791,7 @@ void IssuesWidget::reinitProjectList(const QString &currentProject)
     updateBasicProjectInfo(std::nullopt);
     hideOverlays();
     m_issuesView->showProgressIndicator();
+    setFiltersEnabled(false);
     fetchDashboardAndProjectInfo(onDashboardInfoFetched, currentProject);
 }
 
@@ -767,7 +874,7 @@ static QList<LinkWithColumns> linksForIssue(const std::map<QString, Dto::Any> &i
             it = issueRow.find(line);
             if (it != end && !it->second.isNull()) {
                 QTC_ASSERT(it->second.isDouble(), return);
-                link.targetLine = it->second.getDouble();
+                link.target.line = it->second.getDouble();
                 columns.append(findColumn(it->first));
             }
             links.append({link, columns});
@@ -851,7 +958,7 @@ void IssuesWidget::updateVersionItemsEnabledState()
     if (currentDashboardMode() == DashboardMode::Local) {
         std::optional<Dto::ProjectInfoDto> localInfo = localProjectInfo();
         if (QTC_GUARD(localInfo)) {
-            const int versionCount = localInfo->versions.size();
+            const qsizetype versionCount = localInfo->versions.size();
             QTC_ASSERT(versionCount >= 2 && versionCount <= 3, return);
             QStandardItemModel *model = qobject_cast<QStandardItemModel *>(m_localVersions->model());
             QTC_ASSERT(model, return);
@@ -941,14 +1048,17 @@ void IssuesWidget::updateBasicProjectInfo(const std::optional<Dto::ProjectInfoDt
         button->setCheckable(true);
         connect(button, &QToolButton::clicked, this, [this, prefix = kind.prefix]{
             m_currentPrefix = prefix;
+            settings().defaultIssueKind.setValue(prefix);
+            settings().defaultIssueKind.apply();
             updateNamedFilters();
             fetchTable();
         });
         m_typesButtonGroup->addButton(button, ++buttonId);
         m_typesLayout->addWidget(button);
     }
-    if (auto firstButton = m_typesButtonGroup->button(1))
-        firstButton->setChecked(true);
+    // prefer last issue kind (usually style issues)
+    if (auto lastButton = m_typesButtonGroup->button(buttonId))
+        lastButton->setChecked(true);
 
     GuardLocker lock(m_signalBlocker);
     m_userNames.clear();
@@ -963,8 +1073,7 @@ void IssuesWidget::updateBasicProjectInfo(const std::optional<Dto::ProjectInfoDt
     updateVersionsFromProjectInfo(info);
     m_showFilterHelp->setEnabled(info->issueFilterHelp.has_value());
     std::optional<AxivionVersionInfo> suiteVersionInfo = settings().versionInfo();
-    m_localBuild->setEnabled(!m_currentProject.isEmpty()
-                             && suiteVersionInfo && !suiteVersionInfo->versionNumber.isEmpty());
+    m_localBuild->setEnabled(!hasRunningLocalBuild(m_currentProject));
     checkForLocalBuildAndUpdate();
 }
 
@@ -1016,6 +1125,9 @@ void IssuesWidget::updateAllFilters(const QVariant &namedFilter)
 
 void IssuesWidget::setFiltersEnabled(bool enabled)
 {
+    const QList<QAbstractButton *> buttons = m_typesButtonGroup->buttons();
+    for (auto kindButton : buttons)
+        kindButton->setEnabled(enabled);
     m_addedFilter->setEnabled(enabled);
     m_removedFilter->setEnabled(enabled);
     m_ownerFilter->setEnabled(enabled);
@@ -1035,7 +1147,7 @@ IssueListSearch IssuesWidget::searchFromUi() const
     if (m_versionsStack->currentIndex() == int(DashboardMode::Local)) { // local dashboard
         std::optional<Dto::ProjectInfoDto> localInfo = localProjectInfo();
         QTC_ASSERT(localInfo, return search);
-        const int localVersionsCount = localInfo->versions.size();
+        const qsizetype localVersionsCount = localInfo->versions.size();
         QTC_ASSERT(localVersionsCount <= 3, return search);
         switch (m_localVersions->currentIndex()) {
         case LocalVersions::ReferenceVersion:
@@ -1087,7 +1199,7 @@ void IssuesWidget::fetchTable()
     const auto tableHandler = [this](const Dto::TableInfoDto &dto) {
         m_currentTableInfo.emplace(dto);
     };
-    const auto setupHandler = [this](TaskTree *) {
+    const auto setupHandler = [this] {
         m_totalRowCount = 0;
         m_currentTableInfo.reset();
         m_issuesView->showProgressIndicator();
@@ -1101,6 +1213,7 @@ void IssuesWidget::fetchTable()
         }
         // first time lookup... should we cache and maybe represent old data?
         updateTable();
+        QTC_ASSERT(projectInfo(), m_issuesView->hideProgressIndicator(); return);
         IssueListSearch search = searchFromUi();
         search.computeTotalRowCount = true;
         fetchIssues(dashboardMode, search);
@@ -1131,8 +1244,8 @@ void IssuesWidget::fetchIssues(DashboardMode dashboardMode, const IssueListSearc
     const auto issuesHandler = [this, startRow = search.offset](const Dto::IssueTableDto &dto) {
         addIssues(dto, startRow);
     };
-    const auto setupHandler = [this](TaskTree *) { m_issuesView->showProgressIndicator(); };
-    const auto doneHandler = [this](DoneWith) { m_issuesView->hideProgressIndicator(); };
+    const auto setupHandler = [this] { m_issuesView->showProgressIndicator(); };
+    const auto doneHandler = [this] { m_issuesView->hideProgressIndicator(); };
     m_taskTreeRunner.start(issueTableRecipe(dashboardMode, search, issuesHandler),
                            setupHandler, doneHandler);
 }
@@ -1209,11 +1322,15 @@ void IssuesWidget::requestFocusForIssuesTable()
 
 void IssuesWidget::switchDashboard(bool local)
 {
+    m_localDashBoard->setEnabled(false); // avoid re-trigger while switching
+
     if (local) {
         QTC_ASSERT(!m_currentProject.isEmpty(), return);
-        auto callback = [] { switchDashboardMode(DashboardMode::Local, true); };
+        auto onSuccess = [] { switchDashboardMode(DashboardMode::Local, true); };
+        auto onFail = [this] { m_localDashBoard->setEnabled(true); };
         m_issuesView->showProgressIndicator();
-        startLocalDashboard(m_currentProject, callback);
+        setFiltersEnabled(false);
+        startLocalDashboard(m_currentProject, onSuccess, onFail);
     } else {
         switchDashboardMode(DashboardMode::Global, true);
     }
@@ -1251,7 +1368,7 @@ void IssuesWidget::openFilterHelp()
 void IssuesWidget::checkForLocalBuildAndUpdate()
 {
     checkForLocalBuildResults(m_currentProject, [this] {
-        m_localBuild->setEnabled(true);
+        m_localBuild->setEnabled(!hasRunningLocalBuild(m_currentProject));
         m_localDashBoard->setEnabled(true);
     });
 }
@@ -1264,6 +1381,8 @@ static void loadImage(QPromise<QImage> &promise, const QByteArray &data)
 class LazyImageBrowser : public QTextBrowser
 {
 public:
+    QString projectName() const { return m_projectName; }
+
     QVariant loadResource(int type, const QUrl &name) override
     {
         if (type == QTextDocument::ImageResource) {
@@ -1277,8 +1396,9 @@ public:
         return QTextBrowser::loadResource(type, name);
     }
 
-    void setHtmlAfterCheckingCacheSize(const QString &html)
+    void setHtmlAfterCheckingCacheSize(const QString &html, const QString &projectName)
     {
+        m_projectName = projectName;
         if (m_cachedImagesSize >= 1024 * 1024 * 250) { // if we exceeded 250MB reset the doc
             m_cachedImagesSize = 0;
             setDocument(new QTextDocument(this)); // create a new document to clear resources
@@ -1319,7 +1439,7 @@ private:
                 storage,
                 onGroupSetup(onSetup),
                 If (downloadDataRecipe(dashboardMode, storage)) >> Then {
-                    AsyncTask<QImage>(onImageLoadSetup, onImageLoadDone, CallDoneIf::Success) || successItem
+                    AsyncTask<QImage>(onImageLoadSetup, onImageLoadDone, CallDone::OnSuccess) || successItem
                 },
                 onGroupDone(onDone)
             }
@@ -1327,7 +1447,8 @@ private:
     }
 
     QList<QUrl> m_loadingQueue;
-    TaskTreeRunner m_loaderTaskTree;
+    SingleTaskTreeRunner m_loaderTaskTree;
+    QString m_projectName;
     unsigned int m_cachedImagesSize = 0;
 };
 
@@ -1539,7 +1660,7 @@ public:
     void resetDashboard();
     bool handleContextMenu(bool globalDashboard, const QString &issue, const ItemViewEvent &e);
     bool handleProgressContextMenu(const ItemViewEvent &e);
-    void setIssueDetailsHtml(const QString &html);
+    void setIssueDetailsHtml(const QString &html, const QString &projectName);
     void handleAnchorClicked(const QUrl &url);
     void updateNamedFilters();
     void updateLocalBuildStateFor(const QString &projectName, const QString &state, int percent);
@@ -1743,9 +1864,9 @@ bool AxivionPerspective::handleProgressContextMenu(const ItemViewEvent &e)
     return true;
 }
 
-void AxivionPerspective::setIssueDetailsHtml(const QString &html)
+void AxivionPerspective::setIssueDetailsHtml(const QString &html, const QString &projectName)
 {
-    m_issueDetails->setHtmlAfterCheckingCacheSize(html);
+    m_issueDetails->setHtmlAfterCheckingCacheSize(html, projectName);
 }
 
 void AxivionPerspective::handleAnchorClicked(const QUrl &url)
@@ -1767,9 +1888,13 @@ void AxivionPerspective::handleAnchorClicked(const QUrl &url)
         return;
     Link link;
     if (const QString path = query.queryItemValue("filename", QUrl::FullyDecoded); !path.isEmpty())
-        link.targetFilePath = findFileForIssuePath(FilePath::fromUserInput(path));
+        link.targetFilePath = FilePath::fromUserInput(path);
+    FilePath targetFilePath = mappedPathForLink(link);
+    if (targetFilePath.isEmpty())
+        targetFilePath = requestPathMapping(link, m_issueDetails->projectName());
+    link.targetFilePath = targetFilePath;
     if (const QString line = query.queryItemValue("line"); !line.isEmpty())
-        link.targetLine = line.toInt();
+        link.target.line = line.toInt();
     // column entry is wrong - so, ignore it
     if (link.hasValidTarget() && link.targetFilePath.exists())
         EditorManager::openEditorAt(link);
@@ -1876,10 +2001,10 @@ void showErrorMessage(const QString &errorMessage)
     axivionPerspective()->handleShowErrorMessage(errorMessage);
 }
 
-void updateIssueDetails(const QString &html)
+void updateIssueDetails(const QString &html, const QString &projectName)
 {
     QTC_ASSERT(axivionPerspective(), return);
-    axivionPerspective()->setIssueDetailsHtml(html);
+    axivionPerspective()->setIssueDetailsHtml(html, projectName);
 }
 
 void updateNamedFilters()
