@@ -169,8 +169,9 @@ void LanguageClientManager::clientFinished(Client *client)
     const bool unexpectedFinish = client->state() != Client::Shutdown
                                   && client->state() != Client::ShutdownRequested;
 
-    const QList<TextEditor::TextDocument *> &clientDocs
-        = managerInstance->m_clientForDocument.keys(client);
+    const QList<QPointer<TextEditor::TextDocument>> clientDocs = Utils::transform(
+        managerInstance->m_clientForDocument.keys(client),
+        [](TextEditor::TextDocument *doc) { return QPointer<TextEditor::TextDocument>(doc); });
     if (unexpectedFinish) {
         if (!PluginManager::isShuttingDown()) {
             const bool shouldRestart = client->state() > Client::FailedToInitialize
@@ -178,23 +179,27 @@ void LanguageClientManager::clientFinished(Client *client)
             if (shouldRestart && client->reset()) {
                 qCDebug(Log) << "restart unexpectedly finished client: " << client->name() << client;
                 client->log(
+                    QtMsgType::QtWarningMsg,
                     Tr::tr("Unexpectedly finished. Restarting in %1 seconds.").arg(restartTimeoutS));
                 QTimer::singleShot(restartTimeoutS * 1000, client, [client]() { client->start(); });
-                for (TextEditor::TextDocument *document : clientDocs) {
-                    client->deactivateDocument(document);
-                    if (Core::EditorManager::currentEditor()->document() == document)
+                for (const QPointer<TextEditor::TextDocument> &document : clientDocs) {
+                    if (!document)
+                        continue;
+                    client->deactivateDocument(document.get());
+                    if (Core::EditorManager::currentEditor()->document() == document.get())
                         TextEditor::IOutlineWidgetFactory::updateOutline();
                 }
                 return;
             }
             qCDebug(Log) << "client finished unexpectedly: " << client->name() << client;
-            client->log(Tr::tr("Unexpectedly finished."));
+            client->log(QtMsgType::QtCriticalMsg, Tr::tr("Unexpectedly finished."));
         }
     }
 
     if (unexpectedFinish || !QTC_GUARD(clientDocs.isEmpty())) {
-        for (TextEditor::TextDocument *document : clientDocs)
-            openDocumentWithClient(document, nullptr);
+        for (const QPointer<TextEditor::TextDocument> &document : clientDocs)
+            if (document)
+                openDocumentWithClient(document.get(), nullptr);
     }
 
     deleteClient(client, unexpectedFinish);
@@ -361,7 +366,6 @@ void LanguageClientManager::applySettings(BaseSettings *setting)
         }
     } else if (setting->m_startBehavior == BaseSettings::RequiresProject) {
         const QList<Core::IDocument *> &openedDocuments = Core::DocumentModel::openedDocuments();
-        QHash<Project *, Client *> clientForProject;
         for (Core::IDocument *document : openedDocuments) {
             auto textDocument = qobject_cast<TextEditor::TextDocument *>(document);
             if (!textDocument || !setting->m_languageFilter.isSupported(textDocument))
@@ -369,6 +373,7 @@ void LanguageClientManager::applySettings(BaseSettings *setting)
             const Utils::FilePath filePath = textDocument->filePath();
             for (Project *project : ProjectManager::projects()) {
                 for (Target *target : project->targets()) {
+                    const bool targetIsActive = project->activeTarget() == target;
                     for (BuildConfiguration *bc : target->buildConfigurations()) {
                         if (!setting->isValidOnBuildConfiguration(bc))
                             continue;
@@ -378,16 +383,16 @@ void LanguageClientManager::applySettings(BaseSettings *setting)
                                   && !ProjectSettings(project).disabledSettings().contains(setting->m_id));
                         if (!settingIsEnabled)
                             continue;
-                        if (project->isKnownFile(filePath)) {
-                            Client *client = clientForProject[project];
-                            if (!client) {
-                                client = startClient(setting, bc);
-                                if (!client)
-                                    continue;
-                                clientForProject[project] = client;
-                            }
+                        if (!project->isKnownFile(filePath))
+                            continue;
+                        Client *client = startClient(setting, bc);
+                        if (!client)
+                            continue;
+                        if (targetIsActive && target->activeBuildConfiguration() == bc
+                            && client->activatable()) {
+                            openDocumentWithClient(textDocument, client);
+                        } else
                             client->openDocument(textDocument);
-                        }
                     }
                 }
             }
@@ -631,6 +636,39 @@ static QList<BaseSettings *> sortedSettingsForDocument(Core::IDocument *document
     });
 }
 
+void LanguageClientManager::documentOpenedForProject(
+    TextEditor::TextDocument *textDocument, BaseSettings *setting, const QList<Client *> &clients)
+{
+    const Utils::FilePath &filePath = textDocument->filePath();
+    for (Project *project : ProjectManager::projects()) {
+        // check whether file is part of this project
+        if (!project->isKnownFile(filePath) && !filePath.isChildOf(project->projectDirectory()))
+            continue;
+        for (Target *target : project->targets()) {
+            const bool activateDocument = project->activeTarget() == target;
+            for (BuildConfiguration *bc : target->buildConfigurations()) {
+                // check whether we already have a client running for this project
+                Client *clientForBc
+                    = Utils::findOrDefault(clients, Utils::equal(&Client::buildConfiguration, bc));
+
+                // create a client only when valid on the current project
+                if (!clientForBc) {
+                    if (!setting->isValidOnBuildConfiguration(bc))
+                        continue;
+                    clientForBc = startClient(setting, bc);
+                }
+
+                QTC_ASSERT(clientForBc, continue);
+                if (activateDocument && clientForBc->activatable()
+                    && target->activeBuildConfiguration() == bc) {
+                    openDocumentWithClient(textDocument, clientForBc);
+                } else
+                    clientForBc->openDocument(textDocument);
+            }
+        }
+    }
+}
+
 void LanguageClientManager::documentOpened(Core::IDocument *document)
 {
     auto textDocument = qobject_cast<TextEditor::TextDocument *>(document);
@@ -641,45 +679,28 @@ void LanguageClientManager::documentOpened(Core::IDocument *document)
     const QList<BaseSettings *> settings = sortedSettingsForDocument(document);
     QList<Client *> allClients;
     for (BaseSettings *setting : settings) {
-        QList<Client *> clients = clientsForSetting(setting);
-        if (setting->m_startBehavior == BaseSettings::RequiresProject) {
-            const Utils::FilePath &filePath = document->filePath();
-            for (Project *project : ProjectManager::projects()) {
-                // check whether file is part of this project
-                if (!project->isKnownFile(filePath))
-                    continue;
-                for (Target *target : project->targets()) {
-                    bool activateDocument = project->activeTarget() == target;
-                    for (BuildConfiguration *bc : target->buildConfigurations()) {
-                        // check whether we already have a client running for this project
-                        Client *clientForBc = Utils::findOrDefault(
-                            clients, Utils::equal(&Client::buildConfiguration, bc));
-
-                        // create a client only when valid on the current project
-                        if (!clientForBc) {
-                            if (!setting->isValidOnBuildConfiguration(bc))
-                                continue;
-                            clientForBc = startClient(setting, bc);
-                        }
-
-                        QTC_ASSERT(clientForBc, continue);
-                        activateDocument |= clientForBc->activatable()
-                                            && target->activeBuildConfiguration() == bc;
-                        if (activateDocument)
-                            openDocumentWithClient(textDocument, clientForBc);
-                        else
-                            clientForBc->openDocument(textDocument);
-                        // Since we already opened the document in this client we remove the client
-                        // from the list of clients that receive the openDocument call
-                        clients.removeAll(clientForBc);
-                    }
-                }
+        const QList<Client *> clients = clientsForSetting(setting);
+        switch (setting->m_startBehavior) {
+        case BaseSettings::RequiresProject: {
+            documentOpenedForProject(textDocument, setting, clients);
+            break;
+        }
+        case BaseSettings::RequiresFile: {
+            if (clients.isEmpty()) {
+                Client *client = startClient(setting);
+                QTC_ASSERT(client, break);
+                allClients << client;
+            } else {
+                allClients << clients;
             }
-        } else if (setting->m_startBehavior == BaseSettings::RequiresFile && clients.isEmpty()) {
-            if (Client *client = startClient(setting); QTC_GUARD(client))
-                clients << client;
+            break;
+        }
+        case BaseSettings::AlwaysOn:
+            allClients << clients;
+            break;
+        case BaseSettings::LastSentinel:
+            break;
         }
-        allClients << clients;
     }
 
     for (auto client : std::as_const(allClients)) {
