@@ -11,7 +11,6 @@
 #include <utils/layoutbuilder.h>
 #include <utils/qtcwidgets.h>
 
-#include <QMetaEnum>
 #include <QCompleter>
 
 using namespace Layouting;
@@ -110,6 +109,7 @@ void constructWidget(std::unique_ptr<T> &widget, const sol::table &children)
     };
 // clang-format on
 
+CREATE_HAS_FUNC(onDestroyed, nullptr, nullptr)
 CREATE_HAS_FUNC(onTextChanged, nullptr, nullptr)
 CREATE_HAS_FUNC(onClicked, nullptr, nullptr)
 CREATE_HAS_FUNC(setText, QString())
@@ -130,6 +130,7 @@ CREATE_HAS_FUNC(setWordWrap, bool())
 CREATE_HAS_FUNC(setTextFormat, Qt::TextFormat())
 CREATE_HAS_FUNC(setRightSideIconPath, Utils::FilePath())
 CREATE_HAS_FUNC(setPlaceHolderText, QString())
+CREATE_HAS_FUNC(setPlaceholderText, QString())
 CREATE_HAS_FUNC(setCompleter, nullptr)
 CREATE_HAS_FUNC(setMinimumHeight, int())
 CREATE_HAS_FUNC(onReturnPressed, nullptr, nullptr)
@@ -146,6 +147,7 @@ CREATE_HAS_FUNC(setEnableCodeCopyButton, bool());
 CREATE_HAS_FUNC(setDefaultAction, nullptr);
 CREATE_HAS_FUNC_NAMED(setRole, setRoleButton, QtcButton::Role());
 CREATE_HAS_FUNC_NAMED(setRole, setRoleLabel, QtcLabel::Role());
+CREATE_HAS_FUNC(setCompletionBehavior, Utils::CompletingTextEdit::CompletionBehavior());
 
 template<class T>
 void setProperties(std::unique_ptr<T> &item, const sol::table &children, QObject *guard)
@@ -230,9 +232,17 @@ void setProperties(std::unique_ptr<T> &item, const sol::table &children, QObject
     }
 
     if constexpr (has_setPlaceHolderText<T>) {
-        const auto text = children.get<sol::optional<QString>>("placeHolderText"sv);
+        const auto compatText = children.get<sol::optional<QString>>("placeHolderText"sv);
+        if (compatText)
+            qWarning() << "placeHolderText is deprecated, use placeholderText instead.";
+        const QString text = children.get<sol::optional<QString>>("placeholderText"sv)
+                                 .value_or(compatText.value_or(QString{}));
+        if (!text.isEmpty())
+            item->setPlaceHolderText(text);
+    } else if constexpr (has_setPlaceholderText<T>) {
+        const auto text = children.get<sol::optional<QString>>("placeholderText"sv);
         if (text)
-            item->setPlaceHolderText(*text);
+            item->setPlaceholderText(*text);
     }
 
     if constexpr (has_setCompleter<T>) {
@@ -327,6 +337,18 @@ void setProperties(std::unique_ptr<T> &item, const sol::table &children, QObject
                 });
         }
     }
+
+    if constexpr (has_onDestroyed<T>) {
+        sol::optional<sol::main_function> onDestroyed
+            = children.get<sol::optional<sol::main_function>>("onDestroyed"sv);
+        if (onDestroyed) {
+            item->onDestroyed(guard, [f = *onDestroyed]() {
+                auto res = void_safe_call(f);
+                QTC_CHECK_RESULT(res);
+            });
+        }
+    }
+
     if constexpr (has_onClicked<T>) {
         sol::optional<sol::main_function> onClicked
             = children.get<sol::optional<sol::main_function>>("onClicked"sv);
@@ -392,6 +414,12 @@ void setProperties(std::unique_ptr<T> &item, const sol::table &children, QObject
         if (role)
             item->setRole(*role);
     }
+    if constexpr (has_setCompletionBehavior<T>) {
+        sol::optional<Utils::CompletingTextEdit::CompletionBehavior> completionBehavior = children.get<sol::optional<Utils::CompletingTextEdit::CompletionBehavior>>(
+            "completionBehavior"sv);
+        if (completionBehavior)
+            item->setCompletionBehavior(*completionBehavior);
+    }
 }
 
 template<class T>
@@ -541,40 +569,40 @@ void setupGuiModule()
             "show",
             &Layout::show,
             sol::base_classes,
-            sol::bases<Object, Thing>());
+            sol::bases<Object>());
 
         gui.new_usertype<Form>(
             "Form",
             sol::call_constructor,
             sol::factories(&construct<Form>),
             sol::base_classes,
-            sol::bases<Layout, Object, Thing>());
+            sol::bases<Layout, Object>());
 
         gui.new_usertype<Column>(
             "Column",
             sol::call_constructor,
             sol::factories(&construct<Column>),
             sol::base_classes,
-            sol::bases<Layout, Object, Thing>());
+            sol::bases<Layout, Object>());
 
         gui.new_usertype<Row>(
             "Row",
             sol::call_constructor,
             sol::factories(&construct<Row>),
             sol::base_classes,
-            sol::bases<Layout, Object, Thing>());
+            sol::bases<Layout, Object>());
         gui.new_usertype<Flow>(
             "Flow",
             sol::call_constructor,
             sol::factories(&construct<Flow>),
             sol::base_classes,
-            sol::bases<Layout, Object, Thing>());
+            sol::bases<Layout, Object>());
         gui.new_usertype<Grid>(
             "Grid",
             sol::call_constructor,
             sol::factories(&construct<Grid>),
             sol::base_classes,
-            sol::bases<Layout, Object, Thing>());
+            sol::bases<Layout, Object>());
 
         // Widgets
         gui.new_usertype<PushButton>(
@@ -588,7 +616,7 @@ void setupGuiModule()
             "setIconPath",
             &PushButton::setIconPath,
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
 
         auto qtcButton = gui.new_usertype<Utils::QtcWidgets::Button>(
             "QtcButton",
@@ -603,7 +631,7 @@ void setupGuiModule()
             "setRole",
             &Utils::QtcWidgets::Button::setRole,
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
 
         gui.new_usertype<Utils::QtcWidgets::Switch>(
             "QtcSwitch",
@@ -618,7 +646,7 @@ void setupGuiModule()
             "onClicked",
             &Utils::QtcWidgets::Switch::onClicked,
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
 
         auto qtcLabel = gui.new_usertype<Utils::QtcWidgets::Label>(
             "QtcLabel",
@@ -631,7 +659,20 @@ void setupGuiModule()
             "setRole",
             &Utils::QtcWidgets::Label::setRole,
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
+
+        gui.new_usertype<Utils::QtcWidgets::SearchBox>(
+            "QtcSearchBox",
+            sol::call_constructor,
+            sol::factories([guard](const sol::table &children) {
+                return constructWidgetType<Utils::QtcWidgets::SearchBox>(children, guard);
+            }),
+            "setPlaceholderText",
+            &Utils::QtcWidgets::SearchBox::setPlaceholderText,
+            "setText",
+            &Utils::QtcWidgets::SearchBox::setText,
+            sol::base_classes,
+            sol::bases<Widget, Object>());
 
         gui.new_usertype<Label>(
             "Label",
@@ -642,7 +683,7 @@ void setupGuiModule()
             "text",
             sol::property(&Label::text),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
 
         gui.new_usertype<Layouting::MarkdownBrowser>(
             "MarkdownBrowser",
@@ -654,7 +695,7 @@ void setupGuiModule()
             sol::property(
                 &Layouting::MarkdownBrowser::toMarkdown, &Layouting::MarkdownBrowser::setMarkdown),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
 
         gui.new_usertype<Widget>(
             "Widget",
@@ -679,7 +720,7 @@ void setupGuiModule()
             "toolTip",
             sol::property(&Widget::setToolTip),
             sol::base_classes,
-            sol::bases<Object, Thing>());
+            sol::bases<Object>());
 
         mirrorEnum(gui, QMetaEnum::fromType<Qt::WidgetAttribute>());
         mirrorEnum(gui, QMetaEnum::fromType<Qt::WindowType>());
@@ -699,14 +740,14 @@ void setupGuiModule()
                 return constructWidgetType<Stack>(children, guard);
             }),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
 
         gui.new_usertype<Tab>(
             "Tab",
             sol::call_constructor,
             sol::factories(&constructTab, &constructTabFromTable),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
 
         gui.new_usertype<ScrollArea>(
             "ScrollArea",
@@ -720,7 +761,7 @@ void setupGuiModule()
                     return constructWidgetType<ScrollArea>(children, guard);
                 }),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
 
         gui.new_usertype<TextEdit>(
             "TextEdit",
@@ -731,7 +772,35 @@ void setupGuiModule()
             "markdown",
             sol::property(&TextEdit::markdown),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
+
+        gui.new_usertype<Layouting::CompletingTextEdit>(
+            "QtcTextEdit",
+            sol::call_constructor,
+            sol::factories([guard](const sol::table &children) {
+                return constructWidgetType<Layouting::CompletingTextEdit>(children, guard);
+            }),
+            "CompletionBehavior",
+            gui.create_with(
+                "OnKeyPress", Utils::CompletingTextEdit::CompletionBehavior::OnKeyPress,
+                "OnTextChange", Utils::CompletingTextEdit::CompletionBehavior::OnTextChange),
+            "markdown",
+            sol::property(&Layouting::CompletingTextEdit::markdown, &Layouting::CompletingTextEdit::setMarkdown),
+            "text",
+            sol::property(&Layouting::CompletingTextEdit::text, &Layouting::CompletingTextEdit::setText),
+            "completionBehavior",
+            sol::property(
+                &Layouting::CompletingTextEdit::completionBehavior,
+                &Layouting::CompletingTextEdit::setCompletionBehavior),
+            "placeHolderText",
+            sol::property(&Layouting::CompletingTextEdit::setPlaceHolderText),
+            "rightSideIconPath",
+            sol::property(&Layouting::CompletingTextEdit::setRightSideIconPath),
+            "completer",
+            sol::property(
+                &Layouting::CompletingTextEdit::completer, &Layouting::CompletingTextEdit::setCompleter),
+            sol::base_classes,
+            sol::bases<Widget, Object>());
 
         gui.new_usertype<LineEdit>(
             "LineEdit",
@@ -744,7 +813,7 @@ void setupGuiModule()
             "rightSideIconPath",
             sol::property(&LineEdit::setRightSideIconPath),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
 
         gui.new_usertype<SpinBox>(
             "SpinBox",
@@ -753,13 +822,13 @@ void setupGuiModule()
                 return constructWidgetType<SpinBox>(children, guard);
             }),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
         gui.new_usertype<Splitter>(
             "Splitter",
             sol::call_constructor,
             sol::factories(&constructSplitter),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
         gui.new_usertype<ToolBar>(
             "ToolBar",
             sol::call_constructor,
@@ -767,7 +836,7 @@ void setupGuiModule()
                 return constructWidgetType<ToolBar>(children, guard);
             }),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
         gui.new_usertype<ToolButton>(
             "ToolButton",
             sol::call_constructor,
@@ -775,7 +844,7 @@ void setupGuiModule()
                 return constructWidgetType<ToolButton>(children, guard);
             }),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
         gui.new_usertype<TabWidget>(
             "TabWidget",
             sol::call_constructor,
@@ -783,7 +852,7 @@ void setupGuiModule()
                 return constructTabWidget(children, guard);
             }),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
 
         gui.new_usertype<Group>(
             "Group",
@@ -792,7 +861,7 @@ void setupGuiModule()
                 return constructWidgetType<Group>(children, guard);
             }),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
 
         gui.new_usertype<Spinner>(
             "Spinner",
@@ -805,7 +874,7 @@ void setupGuiModule()
             "decorated",
             sol::property(&Spinner::setDecorated),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
 
         gui.new_usertype<Layouting::IconDisplay>(
             "IconDisplay",
@@ -814,7 +883,7 @@ void setupGuiModule()
                 return constructWidgetType<Layouting::IconDisplay>(children, guard);
             }),
             sol::base_classes,
-            sol::bases<Widget, Object, Thing>());
+            sol::bases<Widget, Object>());
 
         gui["br"] = &br;
         gui["st"] = &st;
@@ -831,3 +900,21 @@ void setupGuiModule()
 }
 
 } // namespace Lua::Internal
+
+template<typename T>
+concept IsObject = std::is_base_of_v<Object, T>;
+
+template<typename TYPE, typename Handler>
+requires IsObject<TYPE>
+sol::optional<TYPE *> sol_lua_check_get(
+    sol::types<TYPE *>, lua_State *L, int index, Handler &&handler, sol::stack::record &tracking)
+{
+    sol::stack::qualified_check_getter<TYPE *> cg{};
+    auto result = cg.get(L, index, std::forward<Handler>(handler), tracking);
+    if (!result) {
+        return sol::nullopt;
+    }
+    if (!(*result)->product())
+        throw sol::error(std::string("Attempt to access a deleted ") + typeid(TYPE).name());
+    return result;
+}
