@@ -1,82 +1,66 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qmlprofilertool.h"
-#include "qmlprofilerstatemanager.h"
-#include "qmlprofilerruncontrol.h"
-#include "qmlprofilerconstants.h"
+
 #include "qmlprofilerattachdialog.h"
-#include "qmlprofilerviewmanager.h"
 #include "qmlprofilerclientmanager.h"
+#include "qmlprofilerconstants.h"
 #include "qmlprofilermodelmanager.h"
-#include "qmlprofilerdetailsrewriter.h"
-#include "qmlprofilernotesmodel.h"
 #include "qmlprofilerrunconfigurationaspect.h"
+#include "qmlprofilerruncontrol.h"
 #include "qmlprofilersettings.h"
-#include "qmlprofilerplugin.h"
+#include "qmlprofilerstatemanager.h"
 #include "qmlprofilertextmark.h"
-
-#include <app/app_version.h>
-
-#include <debugger/debuggericons.h>
-#include <debugger/debuggermainwindow.h>
-#include <debugger/analyzer/analyzermanager.h>
-
-#include <utils/fancymainwindow.h>
-#include <utils/fileinprojectfinder.h>
-#include <utils/qtcassert.h>
-#include <utils/url.h>
-#include <utils/utilsicons.h>
-#include <projectexplorer/environmentaspect.h>
-#include <projectexplorer/projectexplorer.h>
-#include <projectexplorer/project.h>
-#include <projectexplorer/target.h>
-#include <projectexplorer/session.h>
-#include <projectexplorer/kitinformation.h>
-#include <projectexplorer/taskhub.h>
-#include <texteditor/texteditor.h>
+#include "qmlprofilertr.h"
+#include "qmlprofilerviewmanager.h"
 
 #include <coreplugin/actionmanager/command.h>
+#include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/coreconstants.h>
+#include <coreplugin/documentmanager.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/find/findplugin.h>
 #include <coreplugin/icore.h>
+#include <coreplugin/imode.h>
 #include <coreplugin/messagemanager.h>
 #include <coreplugin/helpmanager.h>
 #include <coreplugin/modemanager.h>
 #include <coreplugin/progressmanager/progressmanager.h>
-#include <coreplugin/imode.h>
 
-#include <qtsupport/qtkitinformation.h>
+#include <debugger/analyzer/analyzerutils.h>
+#include <debugger/debuggerconstants.h>
+#include <debugger/debuggericons.h>
+#include <debugger/debuggermainwindow.h>
+
+#include <projectexplorer/buildconfiguration.h>
+#include <projectexplorer/devicesupport/devicekitaspects.h>
+#include <projectexplorer/devicesupport/idevice.h>
+#include <projectexplorer/environmentaspect.h>
+#include <projectexplorer/project.h>
+#include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/projectmanager.h>
+#include <projectexplorer/runcontrol.h>
+#include <projectexplorer/target.h>
+
+#include <qtsupport/qtkitaspect.h>
+
+#include <texteditor/texteditor.h>
+
+#include <utils/fancymainwindow.h>
+#include <utils/fileinprojectfinder.h>
+#include <utils/fileutils.h>
+#include <utils/qtcassert.h>
+#include <utils/stylehelper.h>
+#include <utils/url.h>
+#include <utils/utilsicons.h>
 
 #include <QApplication>
 #include <QDockWidget>
 #include <QElapsedTimer>
 #include <QFileDialog>
-#include <QHBoxLayout>
 #include <QLabel>
 #include <QLineEdit>
 #include <QMenu>
@@ -94,8 +78,7 @@ using namespace QmlProfiler::Constants;
 using namespace ProjectExplorer;
 using namespace Utils;
 
-namespace QmlProfiler {
-namespace Internal {
+namespace QmlProfiler::Internal {
 
 static QmlProfilerTool *m_instance = nullptr;
 
@@ -127,6 +110,12 @@ public:
     QElapsedTimer m_recordingElapsedTime;
 
     bool m_toolBusy = false;
+
+    std::unique_ptr<Core::ActionContainer> m_options;
+    std::unique_ptr<QAction> m_loadQmlTrace;
+    std::unique_ptr<QAction> m_saveQmlTrace;
+    std::unique_ptr<QAction> m_runAction;
+    std::unique_ptr<QAction> m_attachAction;
 };
 
 QmlProfilerTool::QmlProfilerTool()
@@ -190,35 +179,35 @@ QmlProfilerTool::QmlProfilerTool()
 
     d->m_clearButton = new QToolButton;
     d->m_clearButton->setIcon(Utils::Icons::CLEAN_TOOLBAR.icon());
-    d->m_clearButton->setToolTip(tr("Discard data"));
+    d->m_clearButton->setToolTip(Tr::tr("Discard data"));
 
-    connect(d->m_clearButton, &QAbstractButton::clicked, [this](){
+    connect(d->m_clearButton, &QAbstractButton::clicked, [this] {
         if (checkForUnsavedNotes())
             clearData();
     });
 
     d->m_searchButton = new QToolButton;
     d->m_searchButton->setIcon(Utils::Icons::ZOOM_TOOLBAR.icon());
-    d->m_searchButton->setToolTip(tr("Search timeline event notes."));
+    d->m_searchButton->setToolTip(Tr::tr("Search timeline event notes."));
     d->m_searchButton->setEnabled(false);
 
     connect(d->m_searchButton, &QToolButton::clicked, this, &QmlProfilerTool::showTimeLineSearch);
-    connect(d->m_viewContainer, &QmlProfilerViewManager::viewsCreated, this, [this]() {
+    connect(d->m_viewContainer, &QmlProfilerViewManager::viewsCreated, this, [this] {
         d->m_searchButton->setEnabled(d->m_viewContainer->traceView()->isUsable());
     });
 
     d->m_displayFeaturesButton = new QToolButton;
     d->m_displayFeaturesButton->setIcon(Utils::Icons::FILTER.icon());
-    d->m_displayFeaturesButton->setToolTip(tr("Hide or show event categories."));
+    d->m_displayFeaturesButton->setToolTip(Tr::tr("Hide or show event categories."));
     d->m_displayFeaturesButton->setPopupMode(QToolButton::InstantPopup);
-    d->m_displayFeaturesButton->setProperty("noArrow", true);
+    d->m_displayFeaturesButton->setProperty(StyleHelper::C_NO_ARROW, true);
     d->m_displayFeaturesMenu = new QMenu(d->m_displayFeaturesButton);
     d->m_displayFeaturesButton->setMenu(d->m_displayFeaturesMenu);
     connect(d->m_displayFeaturesMenu, &QMenu::triggered,
             this, &QmlProfilerTool::toggleVisibleFeature);
 
     d->m_timeLabel = new QLabel();
-    d->m_timeLabel->setProperty("panelwidget", true);
+    StyleHelper::setPanelWidget(d->m_timeLabel);
     d->m_timeLabel->setIndent(10);
     updateTimeDisplay();
     connect(d->m_timeLabel, &QObject::destroyed, &d->m_recordingTimer, &QTimer::stop);
@@ -250,13 +239,13 @@ QmlProfilerTool::QmlProfilerTool()
     QmlProfilerTextMarkModel *model = d->m_profilerModelManager->textMarkModel();
     if (EditorManager *editorManager = EditorManager::instance()) {
         connect(editorManager, &EditorManager::editorCreated,
-                model, [this, model](Core::IEditor *editor, const QString &fileName) {
+                model, [this, model](Core::IEditor *editor, const FilePath &filePath) {
             Q_UNUSED(editor)
-            model->createMarks(d->m_viewContainer, fileName);
+            model->createMarks(d->m_viewContainer, filePath.toUrlishString());
         });
     }
 
-    auto updateRecordButton = [this]() {
+    auto updateRecordButton = [this] {
         const bool recording =
                 d->m_profilerState->currentState() != QmlProfilerStateManager::AppRunning
                 ? d->m_profilerState->clientRecording() : d->m_profilerState->serverRecording();
@@ -265,7 +254,7 @@ QmlProfilerTool::QmlProfilerTool()
         const static QIcon recordOff = Debugger::Icons::RECORD_OFF.icon();
 
         // update display
-        d->m_recordButton->setToolTip(recording ? tr("Disable Profiling") : tr("Enable Profiling"));
+        d->m_recordButton->setToolTip(recording ? Tr::tr("Disable Profiling") : Tr::tr("Enable Profiling"));
         d->m_recordButton->setIcon(recording ? recordOn : recordOff);
         d->m_recordButton->setChecked(recording);
     };
@@ -277,6 +266,63 @@ QmlProfilerTool::QmlProfilerTool()
     connect(d->m_profilerState, &QmlProfilerStateManager::clientRecordingChanged,
             d->m_recordButton, updateRecordButton);
     updateRecordButton();
+
+
+    const QString description = Tr::tr("The QML Profiler can be used to find performance "
+                                       "bottlenecks in applications using QML.");
+
+    d->m_runAction = std::make_unique<QAction>(Tr::tr("QML Profiler"));
+    d->m_runAction->setToolTip(description);
+    QObject::connect(d->m_runAction.get(), &QAction::triggered,
+                     this, &QmlProfilerTool::profileStartupProject);
+
+    QAction *toolStartAction = startAction();
+    QObject::connect(toolStartAction, &QAction::changed, this, [this, toolStartAction] {
+        d->m_runAction->setEnabled(toolStartAction->isEnabled());
+    });
+
+    d->m_attachAction = std::make_unique<QAction>(Tr::tr("QML Profiler (Attach to Waiting Application)"));
+    d->m_attachAction->setToolTip(description);
+    QObject::connect(d->m_attachAction.get(), &QAction::triggered,
+                     this, &QmlProfilerTool::attachToWaitingApplication);
+
+    d->m_loadQmlTrace = std::make_unique<QAction>(Tr::tr("Load QML Trace"));
+    connect(d->m_loadQmlTrace.get(), &QAction::triggered,
+            this, &QmlProfilerTool::showLoadDialog, Qt::QueuedConnection);
+
+    d->m_saveQmlTrace = std::make_unique<QAction>(Tr::tr("Save QML Trace"));
+    connect(d->m_saveQmlTrace.get(), &QAction::triggered,
+            this, &QmlProfilerTool::showSaveDialog, Qt::QueuedConnection);
+
+    connect(d->m_profilerState, &QmlProfilerStateManager::serverRecordingChanged,
+            this, [this](bool recording) {
+        d->m_loadQmlTrace->setEnabled(!recording);
+    });
+    d->m_loadQmlTrace->setEnabled(!d->m_profilerState->serverRecording());
+
+    connect(d->m_profilerModelManager, &QmlProfilerModelManager::traceChanged,
+            this, [this] {
+        d->m_saveQmlTrace->setEnabled(!d->m_profilerModelManager->isEmpty());
+    });
+    d->m_saveQmlTrace->setEnabled(!d->m_profilerModelManager->isEmpty());
+
+    d->m_options.reset(ActionManager::createMenu("Analyzer.Menu.QMLOptions"));
+    d->m_options->menu()->setTitle(Tr::tr("QML Profiler Options"));
+    d->m_options->menu()->setEnabled(true);
+    ActionContainer *menu = ActionManager::actionContainer(M_DEBUG_ANALYZER);
+
+    menu->addAction(ActionManager::registerAction(d->m_runAction.get(),
+                                                  "QmlProfiler.Internal"),
+                    Debugger::Constants::G_ANALYZER_TOOLS);
+    menu->addAction(ActionManager::registerAction(d->m_attachAction.get(),
+                                                  "QmlProfiler.AttachToWaitingApplication"),
+                    Debugger::Constants::G_ANALYZER_REMOTE_TOOLS);
+
+    menu->addMenu(d->m_options.get(), G_ANALYZER_OPTIONS);
+    d->m_options->addAction(ActionManager::registerAction(d->m_loadQmlTrace.get(),
+                                                       Constants::QmlProfilerLoadActionId));
+    d->m_options->addAction(ActionManager::registerAction(d->m_saveQmlTrace.get(),
+                                                       Constants::QmlProfilerSaveActionId));
 }
 
 QmlProfilerTool::~QmlProfilerTool()
@@ -295,104 +341,54 @@ void QmlProfilerTool::updateRunActions()
 {
     if (d->m_toolBusy) {
         d->m_startAction->setEnabled(false);
-        d->m_startAction->setToolTip(tr("A QML Profiler analysis is still in progress."));
+        d->m_startAction->setToolTip(Tr::tr("A QML Profiler analysis is still in progress."));
         d->m_stopAction->setEnabled(true);
     } else {
-        QString tooltip = tr("Start QML Profiler analysis.");
-        bool canRun = ProjectExplorerPlugin::canRunStartupProject
-                (ProjectExplorer::Constants::QML_PROFILER_RUN_MODE, &tooltip);
-        d->m_startAction->setToolTip(tooltip);
-        d->m_startAction->setEnabled(canRun);
+        const auto canRun = ProjectExplorerPlugin::canRunStartupProject(
+            ProjectExplorer::Constants::QML_PROFILER_RUN_MODE);
+        d->m_startAction->setToolTip(canRun ? Tr::tr("Start QML Profiler analysis.")
+                                            : canRun.error());
+        d->m_startAction->setEnabled(canRun.has_value());
         d->m_stopAction->setEnabled(false);
     }
 }
 
-void QmlProfilerTool::finalizeRunControl(QmlProfilerRunner *runWorker)
+void QmlProfilerTool::finalizeRunControl(RunControl *runControl)
 {
     d->m_toolBusy = true;
-    auto runControl = runWorker->runControl();
-    if (auto aspect = static_cast<QmlProfilerRunConfigurationAspect *>(
-                runControl->aspect(Constants::SETTINGS))) {
-        if (auto settings = static_cast<const QmlProfilerSettings *>(aspect->currentSettings())) {
-            d->m_profilerConnections->setFlushInterval(settings->flushEnabled.value() ?
-                                                           settings->flushInterval.value() : 0);
-            d->m_profilerModelManager->setAggregateTraces(settings->aggregateTraces.value());
+    if (auto aspect = runControl->aspectData<QmlProfilerRunConfigurationAspect>()) {
+        if (auto settings = static_cast<const QmlProfilerSettings *>(aspect->currentSettings)) {
+            d->m_profilerConnections->setFlushInterval(settings->flushEnabled() ?
+                                                           settings->flushInterval() : 0);
+            d->m_profilerModelManager->setAggregateTraces(settings->aggregateTraces());
         }
     }
 
-    auto handleStop = [this, runControl]() {
-        d->m_toolBusy = false;
-        updateRunActions();
-        disconnect(d->m_stopAction, &QAction::triggered, runControl, &RunControl::initiateStop);
-
-        // If we're still trying to connect, stop now.
-        if (d->m_profilerConnections->isConnecting()) {
-            showNonmodalWarning(tr("The application finished before a connection could be "
-                                   "established. No data was loaded."));
-        }
-        d->m_profilerConnections->disconnectFromServer();
-    };
-
-    connect(runControl, &RunControl::stopped, this, handleStop);
-    connect(runControl, &RunControl::finished, this, [this, handleStop] {
-        if (d->m_toolBusy)
-            handleStop();
-    });
-
     connect(d->m_stopAction, &QAction::triggered, runControl, &RunControl::initiateStop);
 
     updateRunActions();
-    runWorker->registerProfilerStateManager(d->m_profilerState);
 
-    //
-    // Initialize m_projectFinder
-    //
-
-    d->m_profilerModelManager->populateFileFinder(runControl->target());
-
-    connect(d->m_profilerConnections, &QmlProfilerClientManager::connectionFailed,
-            runWorker, [this, runWorker]() {
-        auto infoBox = new QMessageBox(ICore::dialogParent());
-        infoBox->setIcon(QMessageBox::Critical);
-        infoBox->setWindowTitle(Core::Constants::IDE_DISPLAY_NAME);
-
-        const int interval = d->m_profilerConnections->retryInterval();
-        const int retries = d->m_profilerConnections->maximumRetries();
-
-        infoBox->setText(QmlProfilerTool::tr("Could not connect to the in-process QML profiler "
-                                             "within %1 s.\n"
-                                             "Do you want to retry and wait %2 s?")
-                         .arg(interval * retries / 1000.0)
-                         .arg(interval * 2 * retries / 1000.0));
-        infoBox->setStandardButtons(QMessageBox::Retry | QMessageBox::Cancel | QMessageBox::Help);
-        infoBox->setDefaultButton(QMessageBox::Retry);
-        infoBox->setModal(true);
-
-        connect(infoBox, &QDialog::finished, runWorker, [this, runWorker, interval](int result) {
-            switch (result) {
-            case QMessageBox::Retry:
-                d->m_profilerConnections->setRetryInterval(interval * 2);
-                d->m_profilerConnections->retryConnect();
-                break;
-            case QMessageBox::Help:
-                HelpManager::showHelpUrl(
-                            "qthelp://org.qt-project.qtcreator/doc/creator-debugging-qml.html");
-                Q_FALLTHROUGH();
-            case QMessageBox::Cancel:
-                // The actual error message has already been logged.
-                QmlProfilerTool::logState(QmlProfilerTool::tr("Failed to connect."));
-                runWorker->cancelProcess();
-                break;
-            }
-        });
-
-        infoBox->show();
-    }, Qt::QueuedConnection); // Queue any connection failures after reportStarted()
-
-    d->m_profilerConnections->connectToServer(runWorker->serverUrl());
+    d->m_profilerModelManager->populateFileFinder(runControl->buildConfiguration());
     d->m_profilerState->setCurrentState(QmlProfilerStateManager::AppRunning);
 }
 
+void QmlProfilerTool::handleStop()
+{
+    if (!d->m_toolBusy)
+        return;
+
+    d->m_toolBusy = false;
+    updateRunActions();
+    disconnect(d->m_stopAction, &QAction::triggered, nullptr, nullptr);
+
+    // If we're still trying to connect, stop now.
+    if (d->m_profilerConnections->isConnecting()) {
+        showNonmodalWarning(Tr::tr("The application finished before a connection could be "
+                                   "established. No data was loaded."));
+    }
+    d->m_profilerConnections->disconnectFromServer();
+}
+
 void QmlProfilerTool::recordingButtonChanged(bool recording)
 {
     // clientRecording is our intention for new sessions. That may differ from the state of the
@@ -419,17 +415,19 @@ void QmlProfilerTool::gotoSourceLocation(const QString &fileUrl, int lineNumber,
     if (lineNumber < 0 || fileUrl.isEmpty())
         return;
 
-    const QString projectFileName = d->m_profilerModelManager->findLocalFile(fileUrl);
+    const auto projectFileName = d->m_profilerModelManager->findLocalFile(fileUrl);
 
-    QFileInfo fileInfo(projectFileName);
-    if (!fileInfo.exists() || !fileInfo.isReadable())
+    if (!projectFileName.exists() || !projectFileName.isReadableFile())
         return;
 
     // The text editors count columns starting with 0, but the ASTs store the
     // location starting with 1, therefore the -1.
-    EditorManager::openEditorAt(
-                projectFileName, lineNumber == 0 ? 1 : lineNumber, columnNumber - 1, Id(),
-                EditorManager::DoNotSwitchToDesignMode | EditorManager::DoNotSwitchToEditMode);
+    EditorManager::openEditorAt({projectFileName,
+                                 lineNumber == 0 ? 1 : lineNumber,
+                                 columnNumber - 1},
+                                Id(),
+                                EditorManager::DoNotSwitchToDesignMode
+                                    | EditorManager::DoNotSwitchToEditMode);
 }
 
 void QmlProfilerTool::updateTimeDisplay()
@@ -451,8 +449,8 @@ void QmlProfilerTool::updateTimeDisplay()
         break;
     }
     QString timeString = QString::number(seconds,'f',1);
-    QString profilerTimeStr = QmlProfilerTool::tr("%1 s").arg(timeString, 6);
-    d->m_timeLabel->setText(tr("Elapsed: %1").arg(profilerTimeStr));
+    QString profilerTimeStr = Tr::tr("%1 s").arg(timeString, 6);
+    d->m_timeLabel->setText(Tr::tr("Elapsed: %1").arg(profilerTimeStr));
 }
 
 void QmlProfilerTool::showTimeLineSearch()
@@ -498,8 +496,9 @@ void QmlProfilerTool::setButtonsEnabled(bool enable)
 void QmlProfilerTool::createInitialTextMarks()
 {
     QmlProfilerTextMarkModel *model = d->m_profilerModelManager->textMarkModel();
-    foreach (IDocument *document, DocumentModel::openedDocuments())
-        model->createMarks(d->m_viewContainer, document->filePath().toString());
+    const QList<IDocument *> documents = DocumentModel::openedDocuments();
+    for (IDocument *document : documents)
+        model->createMarks(d->m_viewContainer, document->filePath().toUrlishString());
 }
 
 bool QmlProfilerTool::prepareTool()
@@ -515,7 +514,7 @@ bool QmlProfilerTool::prepareTool()
     return true;
 }
 
-ProjectExplorer::RunControl *QmlProfilerTool::attachToWaitingApplication()
+RunControl *QmlProfilerTool::attachToWaitingApplication()
 {
     if (!prepareTool())
         return nullptr;
@@ -525,10 +524,10 @@ ProjectExplorer::RunControl *QmlProfilerTool::attachToWaitingApplication()
     Kit *kit = nullptr;
 
     {
-        QSettings *settings = ICore::settings();
+        QtcSettings *settings = ICore::settings();
 
-        kitId = Id::fromSetting(settings->value(QLatin1String("AnalyzerQmlAttachDialog/kitId")));
-        port = settings->value(QLatin1String("AnalyzerQmlAttachDialog/port"), 3768).toInt();
+        kitId = Id::fromSetting(settings->value("AnalyzerQmlAttachDialog/kitId"));
+        port = settings->value("AnalyzerQmlAttachDialog/port", 3768).toInt();
 
         QmlProfilerAttachDialog dialog;
 
@@ -543,13 +542,13 @@ ProjectExplorer::RunControl *QmlProfilerTool::attachToWaitingApplication()
         QTC_ASSERT(port >= 0, return nullptr);
         QTC_ASSERT(port <= std::numeric_limits<quint16>::max(), return nullptr);
 
-        settings->setValue(QLatin1String("AnalyzerQmlAttachDialog/kitId"), kit->id().toSetting());
-        settings->setValue(QLatin1String("AnalyzerQmlAttachDialog/port"), port);
+        settings->setValue("AnalyzerQmlAttachDialog/kitId", kit->id().toSetting());
+        settings->setValue("AnalyzerQmlAttachDialog/port", port);
     }
 
     QUrl serverUrl;
 
-    IDevice::ConstPtr device = DeviceKitAspect::device(kit);
+    IDevice::ConstPtr device = RunDeviceKitAspect::device(kit);
     QTC_ASSERT(device, return nullptr);
     QUrl toolControl = device->toolControlChannel(IDevice::QmlControlChannel);
     serverUrl.setScheme(Utils::urlTcpScheme());
@@ -559,13 +558,14 @@ ProjectExplorer::RunControl *QmlProfilerTool::attachToWaitingApplication()
     d->m_viewContainer->perspective()->select();
 
     auto runControl = new RunControl(ProjectExplorer::Constants::QML_PROFILER_RUN_MODE);
-    runControl->setRunConfiguration(SessionManager::startupRunConfiguration());
-    auto profiler = new QmlProfilerRunner(runControl);
-    profiler->setServerUrl(serverUrl);
+    if (RunConfiguration *runConfig = activeRunConfigForActiveProject())
+        runControl->copyDataFromRunConfiguration(runConfig);
+    runControl->setQmlChannel(serverUrl);
+    runControl->setRunRecipe(qmlProfilerRecipe(runControl));
 
     connect(d->m_profilerConnections, &QmlProfilerClientManager::connectionClosed,
             runControl, &RunControl::initiateStop);
-    ProjectExplorerPlugin::startRunControl(runControl);
+    runControl->start();
     return runControl;
 }
 
@@ -578,7 +578,7 @@ void QmlProfilerTool::showErrorDialog(const QString &error)
 {
     auto errorDialog = new QMessageBox(ICore::dialogParent());
     errorDialog->setIcon(QMessageBox::Warning);
-    errorDialog->setWindowTitle(tr("QML Profiler"));
+    errorDialog->setWindowTitle(Tr::tr("QML Profiler"));
     errorDialog->setText(error);
     errorDialog->setStandardButtons(QMessageBox::Ok);
     errorDialog->setDefaultButton(QMessageBox::Ok);
@@ -586,30 +586,37 @@ void QmlProfilerTool::showErrorDialog(const QString &error)
     errorDialog->show();
 }
 
-void saveLastTraceFile(const QString &filename)
+static void saveLastTraceFile(const FilePath &filePath)
 {
-    QmlProfilerSettings *settings = QmlProfilerPlugin::globalSettings();
-    if (filename != settings->lastTraceFile.value()) {
-        settings->lastTraceFile.setValue(filename);
-        settings->writeGlobalSettings();
+    QmlProfilerSettings &s = globalSettings();
+    if (filePath != s.lastTraceFile()) {
+        s.lastTraceFile.setValue(filePath);
+        s.writeSettings();
     }
 }
 
+static QString traceFileFilters(const QString &tFile, const QString &zFile)
+{
+    QString qmlTraceFiles = Tr::tr("QML traces (*%1 *%2)").arg(zFile, tFile);
+    return qmlTraceFiles.append(";;").append(Core::DocumentManager::allFilesFilterString());
+}
+
 void QmlProfilerTool::showSaveDialog()
 {
     QLatin1String tFile(QtdFileExtension);
     QLatin1String zFile(QztFileExtension);
-    QString filename = QFileDialog::getSaveFileName(
-                ICore::dialogParent(), tr("Save QML Trace"),
-                QmlProfilerPlugin::globalSettings()->lastTraceFile.value(),
-                tr("QML traces (*%1 *%2)").arg(zFile).arg(tFile));
-    if (!filename.isEmpty()) {
-        if (!filename.endsWith(zFile) && !filename.endsWith(tFile))
-            filename += zFile;
-        saveLastTraceFile(filename);
+
+    FilePath filePath = FileUtils::getSaveFilePath(
+                Tr::tr("Save QML Trace"),
+                globalSettings().lastTraceFile(),
+                traceFileFilters(tFile, zFile));
+    if (!filePath.isEmpty()) {
+        if (!filePath.endsWith(zFile) && !filePath.endsWith(tFile))
+            filePath = filePath.stringAppended(zFile);
+        saveLastTraceFile(filePath);
         Debugger::enableMainWindow(false);
-        Core::ProgressManager::addTask(d->m_profilerModelManager->save(filename),
-                                       tr("Saving Trace Data"), TASK_SAVE,
+        Core::ProgressManager::addTask(d->m_profilerModelManager->save(filePath.toUrlishString()),
+                                       Tr::tr("Saving Trace Data"), TASK_SAVE,
                                        Core::ProgressManager::ShowInApplicationIcon);
     }
 }
@@ -623,19 +630,20 @@ void QmlProfilerTool::showLoadDialog()
 
     QLatin1String tFile(QtdFileExtension);
     QLatin1String zFile(QztFileExtension);
-    QString filename = QFileDialog::getOpenFileName(
-                ICore::dialogParent(), tr("Load QML Trace"),
-                QmlProfilerPlugin::globalSettings()->lastTraceFile.value(),
-                tr("QML traces (*%1 *%2)").arg(zFile).arg(tFile));
 
-    if (!filename.isEmpty()) {
-        saveLastTraceFile(filename);
+    FilePath filePath = FileUtils::getOpenFilePath(
+                Tr::tr("Load QML Trace"),
+                globalSettings().lastTraceFile(),
+                traceFileFilters(tFile, zFile));
+
+    if (!filePath.isEmpty()) {
+        saveLastTraceFile(filePath);
         Debugger::enableMainWindow(false);
         connect(d->m_profilerModelManager, &QmlProfilerModelManager::recordedFeaturesChanged,
                 this, &QmlProfilerTool::setRecordedFeatures);
         d->m_profilerModelManager->populateFileFinder();
-        Core::ProgressManager::addTask(d->m_profilerModelManager->load(filename),
-                                       tr("Loading Trace Data"), TASK_LOAD);
+        Core::ProgressManager::addTask(d->m_profilerModelManager->load(filePath.toUrlishString()),
+                                       Tr::tr("Loading Trace Data"), TASK_LOAD);
     }
 }
 
@@ -674,8 +682,8 @@ bool QmlProfilerTool::checkForUnsavedNotes()
 {
     if (!d->m_profilerModelManager->notesModel()->isModified())
         return true;
-    return QMessageBox::warning(QApplication::activeWindow(), tr("QML Profiler"),
-                                tr("You are about to discard the profiling data, including unsaved "
+    return QMessageBox::warning(QApplication::activeWindow(), Tr::tr("QML Profiler"),
+                                Tr::tr("You are about to discard the profiling data, including unsaved "
                                    "notes. Do you want to continue?"),
                                 QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes;
 }
@@ -688,7 +696,7 @@ void QmlProfilerTool::clientsDisconnected()
         } else if (d->m_profilerState->serverRecording()) {
             // If the application stopped by itself, check if we have all the data
             if (d->m_profilerState->currentState() != QmlProfilerStateManager::AppStopRequested) {
-                showNonmodalWarning(tr("Application finished before loading profiled data.\n"
+                showNonmodalWarning(Tr::tr("Application finished before loading profiled data.\n"
                                        "Please use the stop button instead."));
             }
         }
@@ -696,7 +704,7 @@ void QmlProfilerTool::clientsDisconnected()
 
     // ... and return to the "base" state
     if (d->m_profilerState->currentState() == QmlProfilerStateManager::AppDying) {
-        QTimer::singleShot(0, d->m_profilerState, [this]() {
+        QTimer::singleShot(0, d->m_profilerState, [this] {
             d->m_profilerState->setCurrentState(QmlProfilerStateManager::Idle);
         });
     }
@@ -705,7 +713,7 @@ void QmlProfilerTool::clientsDisconnected()
 void addFeatureToMenu(QMenu *menu, ProfileFeature feature, quint64 enabledFeatures)
 {
     QAction *action =
-            menu->addAction(QmlProfilerTool::tr(QmlProfilerModelManager::featureName(feature)));
+            menu->addAction(Tr::tr(QmlProfilerModelManager::featureName(feature)));
     action->setCheckable(true);
     action->setData(static_cast<uint>(feature));
     action->setChecked(enabledFeatures & (1ULL << (feature)));
@@ -731,7 +739,8 @@ void QmlProfilerTool::setAvailableFeatures(quint64 features)
 
 void QmlProfilerTool::setRecordedFeatures(quint64 features)
 {
-    foreach (QAction *action, d->m_displayFeaturesMenu->actions())
+    const QList<QAction *> actions = d->m_displayFeaturesMenu->actions();
+    for (QAction *action : actions)
         action->setEnabled(features & (1ULL << action->data().toUInt()));
 }
 
@@ -771,7 +780,7 @@ void QmlProfilerTool::showNonmodalWarning(const QString &warningMsg)
 {
     auto noExecWarning = new QMessageBox(ICore::dialogParent());
     noExecWarning->setIcon(QMessageBox::Warning);
-    noExecWarning->setWindowTitle(tr("QML Profiler"));
+    noExecWarning->setWindowTitle(Tr::tr("QML Profiler"));
     noExecWarning->setText(warningMsg);
     noExecWarning->setStandardButtons(QMessageBox::Ok);
     noExecWarning->setDefaultButton(QMessageBox::Ok);
@@ -812,7 +821,7 @@ void QmlProfilerTool::profilerStateChanged()
             d->m_profilerConnections->stopRecording();
         } else {
             // Directly transition to idle
-            QTimer::singleShot(0, d->m_profilerState, [this]() {
+            QTimer::singleShot(0, d->m_profilerState, [this] {
                 d->m_profilerState->setCurrentState(QmlProfilerStateManager::Idle);
             });
         }
@@ -831,8 +840,8 @@ void QmlProfilerTool::serverRecordingChanged()
             // We cannot stop it here, so we cannot give the usual yes/no dialog. Show a dialog
             // offering to immediately save the data instead.
             if (d->m_profilerModelManager->notesModel()->isModified() &&
-                    QMessageBox::warning(QApplication::activeWindow(), tr("QML Profiler"),
-                                         tr("Starting a new profiling session will discard the "
+                    QMessageBox::warning(QApplication::activeWindow(), Tr::tr("QML Profiler"),
+                                         Tr::tr("Starting a new profiling session will discard the "
                                             "previous data, including unsaved notes.\nDo you want "
                                             "to save the data first?"),
                                          QMessageBox::Save, QMessageBox::Discard) ==
@@ -874,8 +883,17 @@ void QmlProfilerTool::toggleVisibleFeature(QAction *action)
                     d->m_profilerModelManager->visibleFeatures() | (1ULL << feature));
     else
         d->m_profilerModelManager->setVisibleFeatures(
-                    d->m_profilerModelManager->visibleFeatures() & (~(1ULL << feature)));
+            d->m_profilerModelManager->visibleFeatures() & (~(1ULL << feature)));
 }
 
-} // namespace Internal
-} // namespace QmlProfiler
+void setupQmlProfilerTool()
+{
+    (void) new QmlProfilerTool;
+}
+
+void destroyQmlProfilerTool()
+{
+    delete m_instance;
+}
+
+} // QmlProfiler::Internal
