@@ -1,44 +1,21 @@
-/****************************************************************************
-**
-** Copyright (C) 2017 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2017 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <utils/algorithm.h>
 #include <utils/settingsaccessor.h>
 
 #include <QTemporaryDir>
 
-#include <QtTest>
+#include <QTest>
 
 using namespace Utils;
 
-const char TESTACCESSOR_DN[] = "Test Settings Accessor";
 const char TESTACCESSOR_APPLICATION_DN[] = "SettingsAccessor Test (Basic)";
 const char TESTACCESSOR_DEFAULT_ID[] = "testId";
 
-QVariantMap generateExtraData()
+Store generateExtraData()
 {
-    QVariantMap extra;
+    Store extra;
     extra.insert("Foo", "Bar");
     extra.insert("Int", 42);
     return extra;
@@ -48,17 +25,17 @@ QVariantMap generateExtraData()
 // TestVersionUpgrader:
 // --------------------------------------------------------------------
 
-class TestVersionUpgrader : public Utils::VersionUpgrader
+class TestVersionUpgrader : public VersionUpgrader
 {
 public:
-    TestVersionUpgrader(int version) :
-        Utils::VersionUpgrader(version, QString("v") + QString::number(version))
+    TestVersionUpgrader(int version)
+        : VersionUpgrader(version, "v" + QString::number(version))
     { }
 
-    QVariantMap upgrade(const QVariantMap &data) final
+    Store upgrade(const Store &data) final
     {
-        QVariantMap result = data;
-        result.insert(QString("VERSION_") + QString::number(version()), version());
+        Store result = data;
+        result.insert(numberedKey("VERSION_", version()), version());
         return result;
     }
 };
@@ -67,21 +44,21 @@ public:
 // BasicTestSettingsAccessor:
 // --------------------------------------------------------------------
 
-class BasicTestSettingsAccessor : public Utils::MergingSettingsAccessor
+class BasicTestSettingsAccessor : public MergingSettingsAccessor
 {
 public:
-    BasicTestSettingsAccessor(const Utils::FilePath &baseName = Utils::FilePath::fromString("/foo/bar"),
+    BasicTestSettingsAccessor(const FilePath &baseName = "/foo/bar",
                               const QByteArray &id = QByteArray(TESTACCESSOR_DEFAULT_ID));
 
-    using Utils::MergingSettingsAccessor::addVersionUpgrader;
+    using MergingSettingsAccessor::addVersionUpgrader;
 
-    QHash<Utils::FilePath, QVariantMap> files() const { return m_files; }
-    void addFile(const Utils::FilePath &path, const QVariantMap &data) const { m_files.insert(path, data); }
-    Utils::FilePaths fileNames() const { return m_files.keys(); }
-    QVariantMap fileContents(const Utils::FilePath &path) const { return m_files.value(path); }
+    QHash<FilePath, Store> files() const { return m_files; }
+    void addFile(const FilePath &path, const Store &data) const { m_files.insert(path, data); }
+    FilePaths fileNames() const { return m_files.keys(); }
+    Store fileContents(const FilePath &path) const { return m_files.value(path); }
 
 protected:
-    RestoreData readFile(const Utils::FilePath &path) const override
+    RestoreData readFile(const FilePath &path) const override
     {
         if (!m_files.contains(path))
             return RestoreData("File not found.", "File not found.", Issue::Type::ERROR);
@@ -94,7 +71,7 @@ protected:
     {
         Q_UNUSED(global)
 
-        const QString key = local.key;
+        const Key key = local.key;
         const QVariant main = local.main.value(key);
         const QVariant secondary = local.secondary.value(key);
 
@@ -102,26 +79,25 @@ protected:
             return qMakePair(key, main);
 
         if (main.isNull() && secondary.isNull())
-            return nullopt;
+            return std::nullopt;
         if (!main.isNull())
             return qMakePair(key, main);
         return qMakePair(key, secondary);
     }
 
-    Utils::optional<Issue> writeFile(const Utils::FilePath &path, const QVariantMap &data) const override
+    std::optional<Issue> writeFile(const FilePath &path, const Store &data) const override
     {
         if (data.isEmpty()) {
-            return Issue(QCoreApplication::translate("Utils::SettingsAccessor", "Failed to Write File"),
-                         QCoreApplication::translate("Utils::SettingsAccessor", "There was nothing to write."),
+            return Issue("Failed to Write File", "There was nothing to write.",
                          Issue::Type::WARNING);
         }
 
         addFile(path, data);
-        return nullopt;
+        return std::nullopt;
     }
 
 private:
-    mutable QHash<Utils::FilePath, QVariantMap> m_files;
+    mutable QHash<FilePath, Store> m_files;
 };
 
 // --------------------------------------------------------------------
@@ -130,18 +106,18 @@ private:
 
 class BasicTestSettingsAccessor;
 
-class TestBackUpStrategy : public Utils::VersionedBackUpStrategy
+class TestBackUpStrategy : public VersionedBackUpStrategy
 {
 public:
     TestBackUpStrategy(BasicTestSettingsAccessor *accessor) :
         VersionedBackUpStrategy(accessor)
     { }
 
-    FilePaths readFileCandidates(const Utils::FilePath &baseFileName) const
+    FilePaths readFileCandidates(const FilePath &baseFileName) const override
     {
         return Utils::filtered(static_cast<const BasicTestSettingsAccessor *>(accessor())->fileNames(),
-                               [&baseFileName](const Utils::FilePath &f) {
-            return f.parentDir() == baseFileName.parentDir() && f.toString().startsWith(baseFileName.toString());
+                               [&baseFileName](const FilePath &f) {
+            return f.parentDir() == baseFileName.parentDir() && f.toUrlishString().startsWith(baseFileName.toUrlishString());
         });
     }
 };
@@ -153,7 +129,7 @@ public:
 class TestSettingsAccessor : public BasicTestSettingsAccessor
 {
 public:
-    TestSettingsAccessor(const Utils::FilePath &baseName = Utils::FilePath::fromString("/foo/baz"),
+    TestSettingsAccessor(const FilePath &baseName = "/foo/baz",
                          const QByteArray &id = QByteArray(TESTACCESSOR_DEFAULT_ID)) :
         BasicTestSettingsAccessor(baseName, id)
     {
@@ -163,13 +139,14 @@ public:
     }
 
     // Make methods public for the tests:
-    using Utils::MergingSettingsAccessor::upgradeSettings;
+    using MergingSettingsAccessor::upgradeSettings;
 };
 
-BasicTestSettingsAccessor::BasicTestSettingsAccessor(const FilePath &baseName, const QByteArray &id) :
-    Utils::MergingSettingsAccessor(std::make_unique<TestBackUpStrategy>(this),
-                                   "TestData", TESTACCESSOR_DN, TESTACCESSOR_APPLICATION_DN)
+BasicTestSettingsAccessor::BasicTestSettingsAccessor(const FilePath &baseName, const QByteArray &id)
 {
+    setDocType("TestData");
+    setApplicationDisplayName(TESTACCESSOR_APPLICATION_DN);
+    setStrategy(std::make_unique<TestBackUpStrategy>(this));
     setSettingsId(id);
     setBaseFilePath(baseName);
 }
@@ -223,10 +200,9 @@ private slots:
     void loadSettings_pickBest();
 };
 
-static QVariantMap versionedMap(int version, const QByteArray &id = QByteArray(),
-                                const QVariantMap &extra = QVariantMap())
+static Store versionedMap(int version, const QByteArray &id = {}, const Store &extra = {})
 {
-    QVariantMap result;
+    Store result;
     result.insert("Version", version);
     if (!id.isEmpty())
         result.insert("EnvironmentId", id);
@@ -235,17 +211,17 @@ static QVariantMap versionedMap(int version, const QByteArray &id = QByteArray()
     return result;
 }
 
-static Utils::SettingsAccessor::RestoreData restoreData(const Utils::FilePath &path,
-                                                        const QVariantMap &data)
+static SettingsAccessor::RestoreData restoreData(const FilePath &path,
+                                                        const Store &data)
 {
-    return Utils::SettingsAccessor::RestoreData(path, data);
+    return SettingsAccessor::RestoreData(path, data);
 }
 
-static Utils::SettingsAccessor::RestoreData restoreData(const QByteArray &path,
-                                                        const QVariantMap &data)
-{
-    return restoreData(Utils::FilePath::fromUtf8(path), data);
-}
+//static SettingsAccessor::RestoreData restoreData(const QByteArray &path,
+//                                                        const Store &data)
+//{
+//    return restoreData(FilePath::fromUtf8(path), data);
+//}
 
 void tst_SettingsAccessor::addVersionUpgrader()
 {
@@ -344,8 +320,8 @@ void tst_SettingsAccessor::RestoreDataCompare()
 {
     const TestSettingsAccessor accessor;
 
-    Utils::SettingsAccessor::RestoreData a = restoreData("/foo/bar", versionedMap(5, TESTACCESSOR_DEFAULT_ID));
-    Utils::SettingsAccessor::RestoreData b = restoreData("/foo/baz", versionedMap(6, TESTACCESSOR_DEFAULT_ID));
+    SettingsAccessor::RestoreData a = restoreData("/foo/bar", versionedMap(5, TESTACCESSOR_DEFAULT_ID));
+    SettingsAccessor::RestoreData b = restoreData("/foo/baz", versionedMap(6, TESTACCESSOR_DEFAULT_ID));
 
     QCOMPARE(accessor.strategy()->compare(a, a), 0);
     QCOMPARE(accessor.strategy()->compare(a, b), 1);
@@ -356,8 +332,8 @@ void tst_SettingsAccessor::RestoreDataCompare_idMismatch()
 {
     const TestSettingsAccessor accessor;
 
-    Utils::SettingsAccessor::RestoreData a = restoreData("/foo/bar", versionedMap(5, TESTACCESSOR_DEFAULT_ID));
-    Utils::SettingsAccessor::RestoreData b = restoreData("/foo/baz", versionedMap(6, "foo"));
+    SettingsAccessor::RestoreData a = restoreData("/foo/bar", versionedMap(5, TESTACCESSOR_DEFAULT_ID));
+    SettingsAccessor::RestoreData b = restoreData("/foo/baz", versionedMap(6, "foo"));
 
     QCOMPARE(accessor.strategy()->compare(a, b), -1);
     QCOMPARE(accessor.strategy()->compare(b, a), 1);
@@ -365,10 +341,10 @@ void tst_SettingsAccessor::RestoreDataCompare_idMismatch()
 
 void tst_SettingsAccessor::RestoreDataCompare_noId()
 {
-    const TestSettingsAccessor accessor(Utils::FilePath::fromString("/foo/baz"), QByteArray());
+    const TestSettingsAccessor accessor("/foo/baz", QByteArray());
 
-    Utils::SettingsAccessor::RestoreData a = restoreData("/foo/bar", versionedMap(5, TESTACCESSOR_DEFAULT_ID));
-    Utils::SettingsAccessor::RestoreData b = restoreData("/foo/baz", versionedMap(6, "foo"));
+    SettingsAccessor::RestoreData a = restoreData("/foo/bar", versionedMap(5, TESTACCESSOR_DEFAULT_ID));
+    SettingsAccessor::RestoreData b = restoreData("/foo/baz", versionedMap(6, "foo"));
 
     QCOMPARE(accessor.strategy()->compare(a, b), 1);
     QCOMPARE(accessor.strategy()->compare(b, a), -1);
@@ -378,8 +354,8 @@ void tst_SettingsAccessor::RestoreDataCompare_sameVersion()
 {
     const TestSettingsAccessor accessor;
 
-    Utils::SettingsAccessor::RestoreData a = restoreData("/foo/bar", versionedMap(7, TESTACCESSOR_DEFAULT_ID));
-    Utils::SettingsAccessor::RestoreData b = restoreData("/foo/baz", versionedMap(7, TESTACCESSOR_DEFAULT_ID));
+    SettingsAccessor::RestoreData a = restoreData("/foo/bar", versionedMap(7, TESTACCESSOR_DEFAULT_ID));
+    SettingsAccessor::RestoreData b = restoreData("/foo/baz", versionedMap(7, TESTACCESSOR_DEFAULT_ID));
 
     QCOMPARE(accessor.strategy()->compare(a, b), 0);
     QCOMPARE(accessor.strategy()->compare(b, a), 0);
@@ -389,8 +365,8 @@ void tst_SettingsAccessor::RestoreDataCompare_emptyMap()
 {
     const TestSettingsAccessor accessor;
 
-    Utils::SettingsAccessor::RestoreData a = restoreData("/foo/bar", QVariantMap());
-    Utils::SettingsAccessor::RestoreData b = restoreData("/foo/baz", versionedMap(7, TESTACCESSOR_DEFAULT_ID));
+    SettingsAccessor::RestoreData a = restoreData("/foo/bar", Store());
+    SettingsAccessor::RestoreData b = restoreData("/foo/baz", versionedMap(7, TESTACCESSOR_DEFAULT_ID));
 
     QCOMPARE(accessor.strategy()->compare(a, b), 1);
     QCOMPARE(accessor.strategy()->compare(b, a), -1);
@@ -400,8 +376,8 @@ void tst_SettingsAccessor::RestoreDataCompare_twoEmptyMaps()
 {
     const TestSettingsAccessor accessor;
 
-    Utils::SettingsAccessor::RestoreData a = restoreData("/foo/bar", QVariantMap());
-    Utils::SettingsAccessor::RestoreData b = restoreData("/foo/baz", QVariantMap());
+    SettingsAccessor::RestoreData a = restoreData("/foo/bar", Store());
+    SettingsAccessor::RestoreData b = restoreData("/foo/baz", Store());
 
     QCOMPARE(accessor.strategy()->compare(a, b), 0);
     QCOMPARE(accessor.strategy()->compare(b, a), 0);
@@ -411,11 +387,11 @@ void tst_SettingsAccessor::upgradeSettings_noUpgradeNecessary()
 {
     const TestSettingsAccessor accessor;
     const int startVersion = 8;
-    const Utils::SettingsAccessor::RestoreData input
+    const SettingsAccessor::RestoreData input
             = restoreData(accessor.baseFilePath(),
                           versionedMap(startVersion, TESTACCESSOR_DEFAULT_ID, generateExtraData()));
 
-    const Utils::SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 8);
+    const SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 8);
 
     QVERIFY(!result.hasIssue());
     for (auto it = result.data.cbegin(); it != result.data.cend(); ++it) {
@@ -433,12 +409,12 @@ void tst_SettingsAccessor::upgradeSettings_invalidId()
 {
     const TestSettingsAccessor accessor;
     const int startVersion = 8;
-    const Utils::SettingsAccessor::RestoreData input
+    const SettingsAccessor::RestoreData input
             = restoreData(accessor.baseFilePath(),
                           versionedMap(startVersion, "foo", generateExtraData()));
 
 
-    const Utils::SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 8);
+    const SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 8);
 
     // Data is unchanged
     QVERIFY(result.hasWarning());
@@ -457,11 +433,11 @@ void tst_SettingsAccessor::upgradeSettings_tooOld()
 {
     const TestSettingsAccessor accessor;
     const int startVersion = 1;
-    const Utils::SettingsAccessor::RestoreData input
+    const SettingsAccessor::RestoreData input
             = restoreData(accessor.baseFilePath(),
                           versionedMap(startVersion, TESTACCESSOR_DEFAULT_ID, generateExtraData()));
 
-    const Utils::SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 8);
+    const SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 8);
 
     // Data is unchanged
     QVERIFY(result.hasIssue());
@@ -472,11 +448,11 @@ void tst_SettingsAccessor::upgradeSettings_tooNew()
 {
     const TestSettingsAccessor accessor;
     const int startVersion = 42;
-    const Utils::SettingsAccessor::RestoreData input
+    const SettingsAccessor::RestoreData input
             = restoreData(accessor.baseFilePath(),
                           versionedMap(startVersion, TESTACCESSOR_DEFAULT_ID, generateExtraData()));
 
-    const Utils::SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 8);
+    const SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 8);
 
     // Data is unchanged
     QVERIFY(result.hasIssue());
@@ -487,11 +463,11 @@ void tst_SettingsAccessor::upgradeSettings_oneStep()
 {
     const TestSettingsAccessor accessor;
     const int startVersion = 7;
-    const Utils::SettingsAccessor::RestoreData input
+    const SettingsAccessor::RestoreData input
             = restoreData(accessor.baseFilePath(),
                           versionedMap(startVersion, TESTACCESSOR_DEFAULT_ID, generateExtraData()));
 
-    const Utils::SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 8);
+    const SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 8);
 
     QVERIFY(!result.hasIssue());
     for (auto it = result.data.cbegin(); it != result.data.cend(); ++it) {
@@ -513,12 +489,12 @@ void tst_SettingsAccessor::upgradeSettings_twoSteps()
 {
     const TestSettingsAccessor accessor;
     const int startVersion = 6;
-    const Utils::SettingsAccessor::RestoreData input
+    const SettingsAccessor::RestoreData input
             = restoreData(accessor.baseFilePath(),
                           versionedMap(startVersion, TESTACCESSOR_DEFAULT_ID, generateExtraData()));
 
 
-    const Utils::SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 8);
+    const SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 8);
 
     QVERIFY(!result.hasIssue());
     for (auto it = result.data.cbegin(); it != result.data.cend(); ++it) {
@@ -542,11 +518,11 @@ void tst_SettingsAccessor::upgradeSettings_partialUpdate()
 {
     const TestSettingsAccessor accessor;
     const int startVersion = 6;
-    const Utils::SettingsAccessor::RestoreData input
+    const SettingsAccessor::RestoreData input
             = restoreData(accessor.baseFilePath(),
                           versionedMap(startVersion, TESTACCESSOR_DEFAULT_ID, generateExtraData()));
 
-    const Utils::SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 7);
+    const SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 7);
 
     QVERIFY(!result.hasIssue());
     for (auto it = result.data.cbegin(); it != result.data.cend(); ++it) {
@@ -567,13 +543,13 @@ void tst_SettingsAccessor::upgradeSettings_partialUpdate()
 void tst_SettingsAccessor::upgradeSettings_targetVersionTooOld()
 {
     const TestSettingsAccessor accessor;
-    const QVariantMap extra = generateExtraData();
+    const Store extra = generateExtraData();
     const int startVersion = 6;
-    const Utils::SettingsAccessor::RestoreData input
+    const SettingsAccessor::RestoreData input
             = restoreData(accessor.baseFilePath(),
                           versionedMap(startVersion, TESTACCESSOR_DEFAULT_ID, extra));
 
-    const Utils::SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 2);
+    const SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 2);
 
     // result is unchanged!
     QVERIFY(!result.hasIssue());
@@ -583,13 +559,13 @@ void tst_SettingsAccessor::upgradeSettings_targetVersionTooOld()
 void tst_SettingsAccessor::upgradeSettings_targetVersionTooNew()
 {
     const TestSettingsAccessor accessor;
-    const QVariantMap extra = generateExtraData();
+    const Store extra = generateExtraData();
     const int startVersion = 6;
-    const Utils::SettingsAccessor::RestoreData input
+    const SettingsAccessor::RestoreData input
             = restoreData(accessor.baseFilePath(),
                           versionedMap(startVersion, TESTACCESSOR_DEFAULT_ID, extra));
 
-    const Utils::SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 42);
+    const SettingsAccessor::RestoreData result = accessor.upgradeSettings(input, 42);
 
     // result is unchanged!
     QVERIFY(!result.hasIssue());
@@ -601,10 +577,10 @@ void tst_SettingsAccessor::upgradeSettings_targetVersionTooNew()
 void tst_SettingsAccessor::findIssues_ok()
 {
     const TestSettingsAccessor accessor;
-    const QVariantMap data = versionedMap(6, TESTACCESSOR_DEFAULT_ID);
-    const Utils::FilePath path = Utils::FilePath::fromString("/foo/baz.user");
+    const Store data = versionedMap(6, TESTACCESSOR_DEFAULT_ID);
+    const FilePath path = "/foo/baz.user";
 
-    const Utils::optional<Utils::SettingsAccessor::Issue> info = accessor.findIssues(data, path);
+    const std::optional<SettingsAccessor::Issue> info = accessor.findIssues(data, path);
 
     QVERIFY(!info);
 }
@@ -612,10 +588,10 @@ void tst_SettingsAccessor::findIssues_ok()
 void tst_SettingsAccessor::findIssues_emptyData()
 {
     const TestSettingsAccessor accessor;
-    const QVariantMap data;
-    const Utils::FilePath path = Utils::FilePath::fromString("/foo/bar.user");
+    const Store data;
+    const FilePath path = "/foo/bar.user";
 
-    const Utils::optional<Utils::SettingsAccessor::Issue> info = accessor.findIssues(data, path);
+    const std::optional<SettingsAccessor::Issue> info = accessor.findIssues(data, path);
 
     QVERIFY(bool(info));
 }
@@ -623,10 +599,10 @@ void tst_SettingsAccessor::findIssues_emptyData()
 void tst_SettingsAccessor::findIssues_tooNew()
 {
     const TestSettingsAccessor accessor;
-    const QVariantMap data = versionedMap(42, TESTACCESSOR_DEFAULT_ID);
-    const Utils::FilePath path = Utils::FilePath::fromString("/foo/bar.user");
+    const Store data = versionedMap(42, TESTACCESSOR_DEFAULT_ID);
+    const FilePath path = "/foo/bar.user";
 
-    const Utils::optional<Utils::SettingsAccessor::Issue> info = accessor.findIssues(data, path);
+    const std::optional<SettingsAccessor::Issue> info = accessor.findIssues(data, path);
 
     QVERIFY(bool(info));
 }
@@ -634,10 +610,10 @@ void tst_SettingsAccessor::findIssues_tooNew()
 void tst_SettingsAccessor::findIssues_tooOld()
 {
     const TestSettingsAccessor accessor;
-    const QVariantMap data = versionedMap(2, TESTACCESSOR_DEFAULT_ID);
-    const Utils::FilePath path = Utils::FilePath::fromString("/foo/bar.user");
+    const Store data = versionedMap(2, TESTACCESSOR_DEFAULT_ID);
+    const FilePath path = "/foo/bar.user";
 
-    const Utils::optional<Utils::SettingsAccessor::Issue> info = accessor.findIssues(data, path);
+    const std::optional<SettingsAccessor::Issue> info = accessor.findIssues(data, path);
 
     QVERIFY(bool(info));
 }
@@ -645,10 +621,10 @@ void tst_SettingsAccessor::findIssues_tooOld()
 void tst_SettingsAccessor::findIssues_wrongId()
 {
     const TestSettingsAccessor accessor;
-    const QVariantMap data = versionedMap(6, "foo");
-    const Utils::FilePath path = Utils::FilePath::fromString("/foo/bar.user");
+    const Store data = versionedMap(6, "foo");
+    const FilePath path = "/foo/bar.user";
 
-    const Utils::optional<Utils::SettingsAccessor::Issue> info = accessor.findIssues(data, path);
+    const std::optional<SettingsAccessor::Issue> info = accessor.findIssues(data, path);
 
     QVERIFY(bool(info));
 }
@@ -656,10 +632,10 @@ void tst_SettingsAccessor::findIssues_wrongId()
 void tst_SettingsAccessor::findIssues_nonDefaultPath()
 {
     const TestSettingsAccessor accessor;
-    const QVariantMap data = versionedMap(6, TESTACCESSOR_DEFAULT_ID);
-    const Utils::FilePath path = Utils::FilePath::fromString("/foo/bar.user.foobar");
+    const Store data = versionedMap(6, TESTACCESSOR_DEFAULT_ID);
+    const FilePath path = "/foo/bar.user.foobar";
 
-    const Utils::optional<Utils::SettingsAccessor::Issue> info = accessor.findIssues(data, path);
+    const std::optional<SettingsAccessor::Issue> info = accessor.findIssues(data, path);
 
     QVERIFY(bool(info));
 }
@@ -667,14 +643,14 @@ void tst_SettingsAccessor::findIssues_nonDefaultPath()
 
 void tst_SettingsAccessor::saveSettings()
 {
-    const Utils::FilePath baseFile = Utils::FilePath::fromString("/tmp/foo/saveSettings");
+    const FilePath baseFile = "/tmp/foo/saveSettings";
     const TestSettingsAccessor accessor(baseFile);
-    const QVariantMap data = versionedMap(6, TESTACCESSOR_DEFAULT_ID);
+    const Store data = versionedMap(6, TESTACCESSOR_DEFAULT_ID);
 
-    QVERIFY(accessor.saveSettings(data, nullptr));
+    QVERIFY(accessor.saveSettings(data));
 
     QCOMPARE(accessor.files().count(), 1);
-    const QVariantMap read = accessor.fileContents(baseFile);
+    const Store read = accessor.fileContents(baseFile);
 
     QVERIFY(!read.isEmpty());
     for (auto it = read.cbegin(); it != read.cend(); ++it) {
@@ -690,13 +666,13 @@ void tst_SettingsAccessor::saveSettings()
 
 void tst_SettingsAccessor::loadSettings()
 {
-    const QVariantMap data = versionedMap(6, "loadSettings", generateExtraData());
-    const Utils::FilePath path = Utils::FilePath::fromString("/tmp/foo/loadSettings");
+    const Store data = versionedMap(6, "loadSettings", generateExtraData());
+    const FilePath path = "/tmp/foo/loadSettings";
     const TestSettingsAccessor accessor(path, "loadSettings");
     accessor.addFile(path, data);
     QCOMPARE(accessor.files().count(), 1); // Catch changes early:-)
 
-    const QVariantMap read = accessor.restoreSettings(nullptr);
+    const Store read = accessor.restoreSettings();
     QCOMPARE(accessor.files().count(), 1); // no files were created
 
     QVERIFY(!read.isEmpty());
@@ -719,21 +695,21 @@ void tst_SettingsAccessor::loadSettings()
 
 void tst_SettingsAccessor::loadSettings_pickBest()
 {
-    const Utils::FilePath path = Utils::FilePath::fromString("/tmp/foo/loadSettings");
+    const FilePath path = "/tmp/foo/loadSettings";
     const TestSettingsAccessor accessor(path, "loadSettings");
 
     accessor.addFile(path, versionedMap(10, "loadSettings", generateExtraData())); // too new
-    const QVariantMap data = versionedMap(7, "loadSettings", generateExtraData());
-    accessor.addFile(Utils::FilePath::fromString("/tmp/foo/loadSettings.foo"), data); // pick this!
-    accessor.addFile(Utils::FilePath::fromString("/tmp/foo/loadSettings.foo1"),
+    const Store data = versionedMap(7, "loadSettings", generateExtraData());
+    accessor.addFile("/tmp/foo/loadSettings.foo", data); // pick this!
+    accessor.addFile("/tmp/foo/loadSettings.foo1",
                      versionedMap(8, "fooSettings", generateExtraData())); // wrong environment
-    accessor.addFile(Utils::FilePath::fromString("/tmp/foo/loadSettings.bar"),
+    accessor.addFile("/tmp/foo/loadSettings.bar",
                      versionedMap(6, "loadSettings", generateExtraData())); // too old
-    accessor.addFile(Utils::FilePath::fromString("/tmp/foo/loadSettings.baz"),
+    accessor.addFile("/tmp/foo/loadSettings.baz",
                      versionedMap(1, "loadSettings", generateExtraData())); // much too old
     QCOMPARE(accessor.files().count(), 5); // Catch changes early:-)
 
-    const QVariantMap read = accessor.restoreSettings(nullptr);
+    const Store read = accessor.restoreSettings();
     QCOMPARE(accessor.files().count(), 5); // no new files
 
     QVERIFY(!read.isEmpty());
@@ -752,6 +728,6 @@ void tst_SettingsAccessor::loadSettings_pickBest()
     QCOMPARE(read.size(), data.size() + 2);
 }
 
-QTEST_MAIN(tst_SettingsAccessor)
+QTEST_GUILESS_MAIN(tst_SettingsAccessor)
 
 #include "tst_settings.moc"
